// A Bison parser, made by GNU Bison 3.8.2.

// Skeleton implementation for Bison LALR(1) parsers in C++

// Copyright (C) 2002-2015, 2018-2021 Free Software Foundation, Inc.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// As a special exception, you may create a larger work that contains
// part or all of the Bison parser skeleton and distribute that work
// under terms of your choice, so long as that work isn't itself a
// parser generator using the skeleton or a modified version thereof
// as a parser skeleton.  Alternatively, if you modify or redistribute
// the parser skeleton itself, you may (at your option) remove this
// special exception, which will cause the skeleton and the resulting
// Bison output files to be licensed under the GNU General Public
// License without this special exception.

// This special exception was added by the Free Software Foundation in
// version 2.2 of Bison.

// DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
// especially those whose name start with YY_ or yy_.  They are
// private implementation details that can be changed or removed.





#include "parser.hpp"


// Unqualified %code blocks.
#line 45 "src/asm/parser.y"

	#include <algorithm>
	#include <ctype.h>
	#include <inttypes.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <string_view>

	#include "asm/charmap.hpp"
	#include "asm/fixpoint.hpp"
	#include "asm/format.hpp"
	#include "asm/fstack.hpp"
	#include "asm/main.hpp"
	#include "asm/opt.hpp"
	#include "asm/output.hpp"
	#include "asm/section.hpp"
	#include "asm/symbol.hpp"
	#include "asm/warning.hpp"

	#include "extern/utf8decoder.hpp"

	#include "helpers.hpp"

	using namespace std::literals;

	yy::parser::symbol_type yylex(); // Provided by lexer.cpp

	static uint32_t str2int2(std::vector<uint8_t> const &s);
	static void errorInvalidUTF8Byte(uint8_t byte, char const *functionName);
	static size_t strlenUTF8(std::string const &str);
	static std::string strsubUTF8(std::string const &str, uint32_t pos, uint32_t len);
	static size_t charlenUTF8(std::string const &str);
	static std::string charsubUTF8(std::string const &str, uint32_t pos);
	static uint32_t adjustNegativePos(int32_t pos, size_t len, char const *functionName);
	static std::string strrpl(
	    std::string_view str, std::string const &old, std::string const &rep
	);
	static std::string strfmt(
	    std::string const &spec,
	    std::vector<std::variant<uint32_t, std::string>> const &args
	);
	static void compoundAssignment(std::string const &symName, RPNCommand op, int32_t constValue);
	static void failAssert(AssertionType type);
	static void failAssertMsg(AssertionType type, std::string const &message);

	// The CPU encodes instructions in a logical way, so most instructions actually follow patterns.
	// These enums thus help with bit twiddling to compute opcodes.
	enum { REG_B, REG_C, REG_D, REG_E, REG_H, REG_L, REG_HL_IND, REG_A };

	enum { REG_BC_IND, REG_DE_IND, REG_HL_INDINC, REG_HL_INDDEC };

	// REG_AF == REG_SP since LD/INC/ADD/DEC allow SP, while PUSH/POP allow AF
	enum { REG_BC, REG_DE, REG_HL, REG_SP, REG_AF = REG_SP };

	// CC_NZ == CC_Z ^ 1, and CC_NC == CC_C ^ 1, so `!` can toggle them
	enum { CC_NZ, CC_Z, CC_NC, CC_C };

#line 105 "src/asm/parser.cpp"


#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> // FIXME: INFRINGES ON USER NAME SPACE.
#   define YY_(msgid) dgettext ("bison-runtime", msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(msgid) msgid
# endif
#endif


// Whether we are compiled with exception support.
#ifndef YY_EXCEPTIONS
# if defined __GNUC__ && !defined __EXCEPTIONS
#  define YY_EXCEPTIONS 0
# else
#  define YY_EXCEPTIONS 1
# endif
#endif



// Enable debugging if requested.
#if YYDEBUG

// A pseudo ostream that takes yydebug_ into account.
# define YYCDEBUG if (yydebug_) (*yycdebug_)

# define YY_SYMBOL_PRINT(Title, Symbol)         \
  do {                                          \
    if (yydebug_)                               \
    {                                           \
      *yycdebug_ << Title << ' ';               \
      yy_print_ (*yycdebug_, Symbol);           \
      *yycdebug_ << '\n';                       \
    }                                           \
  } while (false)

# define YY_REDUCE_PRINT(Rule)          \
  do {                                  \
    if (yydebug_)                       \
      yy_reduce_print_ (Rule);          \
  } while (false)

# define YY_STACK_PRINT()               \
  do {                                  \
    if (yydebug_)                       \
      yy_stack_print_ ();                \
  } while (false)

#else // !YYDEBUG

# define YYCDEBUG if (false) std::cerr
# define YY_SYMBOL_PRINT(Title, Symbol)  YY_USE (Symbol)
# define YY_REDUCE_PRINT(Rule)           static_cast<void> (0)
# define YY_STACK_PRINT()                static_cast<void> (0)

#endif // !YYDEBUG

#define yyerrok         (yyerrstatus_ = 0)
#define yyclearin       (yyla.clear ())

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab
#define YYRECOVERING()  (!!yyerrstatus_)

namespace yy {
#line 178 "src/asm/parser.cpp"

  /// Build a parser object.
  parser::parser ()
#if YYDEBUG
    : yydebug_ (false),
      yycdebug_ (&std::cerr),
#else
    :
#endif
      yy_lac_established_ (false)
  {}

  parser::~parser ()
  {}

  parser::syntax_error::~syntax_error () YY_NOEXCEPT YY_NOTHROW
  {}

  /*---------.
  | symbol.  |
  `---------*/



  // by_state.
  parser::by_state::by_state () YY_NOEXCEPT
    : state (empty_state)
  {}

  parser::by_state::by_state (const by_state& that) YY_NOEXCEPT
    : state (that.state)
  {}

  void
  parser::by_state::clear () YY_NOEXCEPT
  {
    state = empty_state;
  }

  void
  parser::by_state::move (by_state& that)
  {
    state = that.state;
    that.clear ();
  }

  parser::by_state::by_state (state_type s) YY_NOEXCEPT
    : state (s)
  {}

  parser::symbol_kind_type
  parser::by_state::kind () const YY_NOEXCEPT
  {
    if (state == empty_state)
      return symbol_kind::S_YYEMPTY;
    else
      return YY_CAST (symbol_kind_type, yystos_[+state]);
  }

  parser::stack_symbol_type::stack_symbol_type ()
  {}

  parser::stack_symbol_type::stack_symbol_type (YY_RVREF (stack_symbol_type) that)
    : super_type (YY_MOVE (that.state))
  {
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.YY_MOVE_OR_COPY< AlignmentSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.YY_MOVE_OR_COPY< AssertionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.YY_MOVE_OR_COPY< Capture > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.YY_MOVE_OR_COPY< Expression > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_for_args: // for_args
        value.YY_MOVE_OR_COPY< ForArgs > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.YY_MOVE_OR_COPY< RPNCommand > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.YY_MOVE_OR_COPY< SectionModifier > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.YY_MOVE_OR_COPY< SectionSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.YY_MOVE_OR_COPY< StrFmtArgList > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_const: // const
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_const_8bit: // const_8bit
      case symbol_kind::S_opt_q_arg: // opt_q_arg
      case symbol_kind::S_sect_type: // sect_type
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.YY_MOVE_OR_COPY< int32_t > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.YY_MOVE_OR_COPY< std::shared_ptr<MacroArgs> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.YY_MOVE_OR_COPY< std::string > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.YY_MOVE_OR_COPY< std::vector<Expression> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.YY_MOVE_OR_COPY< std::vector<std::string> > (YY_MOVE (that.value));
        break;

      default:
        break;
    }

#if 201103L <= YY_CPLUSPLUS
    // that is emptied.
    that.state = empty_state;
#endif
  }

  parser::stack_symbol_type::stack_symbol_type (state_type s, YY_MOVE_REF (symbol_type) that)
    : super_type (s)
  {
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.move< AlignmentSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.move< AssertionType > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.move< Capture > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.move< Expression > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_for_args: // for_args
        value.move< ForArgs > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.move< RPNCommand > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.move< SectionModifier > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.move< SectionSpec > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.move< StrFmtArgList > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_const: // const
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_const_8bit: // const_8bit
      case symbol_kind::S_opt_q_arg: // opt_q_arg
      case symbol_kind::S_sect_type: // sect_type
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.move< int32_t > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.move< std::shared_ptr<MacroArgs> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.move< std::string > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.move< std::vector<Expression> > (YY_MOVE (that.value));
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.move< std::vector<std::string> > (YY_MOVE (that.value));
        break;

      default:
        break;
    }

    // that is emptied.
    that.kind_ = symbol_kind::S_YYEMPTY;
  }

#if YY_CPLUSPLUS < 201103L
  parser::stack_symbol_type&
  parser::stack_symbol_type::operator= (const stack_symbol_type& that)
  {
    state = that.state;
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.copy< AlignmentSpec > (that.value);
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.copy< AssertionType > (that.value);
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.copy< Capture > (that.value);
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.copy< Expression > (that.value);
        break;

      case symbol_kind::S_for_args: // for_args
        value.copy< ForArgs > (that.value);
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.copy< RPNCommand > (that.value);
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.copy< SectionModifier > (that.value);
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.copy< SectionSpec > (that.value);
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.copy< StrFmtArgList > (that.value);
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_const: // const
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_const_8bit: // const_8bit
      case symbol_kind::S_opt_q_arg: // opt_q_arg
      case symbol_kind::S_sect_type: // sect_type
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.copy< int32_t > (that.value);
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.copy< std::shared_ptr<MacroArgs> > (that.value);
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.copy< std::string > (that.value);
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.copy< std::vector<Expression> > (that.value);
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.copy< std::vector<std::string> > (that.value);
        break;

      default:
        break;
    }

    return *this;
  }

  parser::stack_symbol_type&
  parser::stack_symbol_type::operator= (stack_symbol_type& that)
  {
    state = that.state;
    switch (that.kind ())
    {
      case symbol_kind::S_align_spec: // align_spec
        value.move< AlignmentSpec > (that.value);
        break;

      case symbol_kind::S_assert_type: // assert_type
        value.move< AssertionType > (that.value);
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        value.move< Capture > (that.value);
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        value.move< Expression > (that.value);
        break;

      case symbol_kind::S_for_args: // for_args
        value.move< ForArgs > (that.value);
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        value.move< RPNCommand > (that.value);
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        value.move< SectionModifier > (that.value);
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        value.move< SectionSpec > (that.value);
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        value.move< StrFmtArgList > (that.value);
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_const: // const
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_const_8bit: // const_8bit
      case symbol_kind::S_opt_q_arg: // opt_q_arg
      case symbol_kind::S_sect_type: // sect_type
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        value.move< int32_t > (that.value);
        break;

      case symbol_kind::S_macro_args: // macro_args
        value.move< std::shared_ptr<MacroArgs> > (that.value);
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        value.move< std::string > (that.value);
        break;

      case symbol_kind::S_ds_args: // ds_args
        value.move< std::vector<Expression> > (that.value);
        break;

      case symbol_kind::S_purge_args: // purge_args
        value.move< std::vector<std::string> > (that.value);
        break;

      default:
        break;
    }

    // that is emptied.
    that.state = empty_state;
    return *this;
  }
#endif

  template <typename Base>
  void
  parser::yy_destroy_ (const char* yymsg, basic_symbol<Base>& yysym) const
  {
    if (yymsg)
      YY_SYMBOL_PRINT (yymsg, yysym);
  }

#if YYDEBUG
  template <typename Base>
  void
  parser::yy_print_ (std::ostream& yyo, const basic_symbol<Base>& yysym) const
  {
    std::ostream& yyoutput = yyo;
    YY_USE (yyoutput);
    if (yysym.empty ())
      yyo << "empty symbol";
    else
      {
        symbol_kind_type yykind = yysym.kind ();
        yyo << (yykind < YYNTOKENS ? "token" : "nterm")
            << ' ' << yysym.name () << " (";
        YY_USE (yykind);
        yyo << ')';
      }
  }
#endif

  void
  parser::yypush_ (const char* m, YY_MOVE_REF (stack_symbol_type) sym)
  {
    if (m)
      YY_SYMBOL_PRINT (m, sym);
    yystack_.push (YY_MOVE (sym));
  }

  void
  parser::yypush_ (const char* m, state_type s, YY_MOVE_REF (symbol_type) sym)
  {
#if 201103L <= YY_CPLUSPLUS
    yypush_ (m, stack_symbol_type (s, std::move (sym)));
#else
    stack_symbol_type ss (s, sym);
    yypush_ (m, ss);
#endif
  }

  void
  parser::yypop_ (int n) YY_NOEXCEPT
  {
    yystack_.pop (n);
  }

#if YYDEBUG
  std::ostream&
  parser::debug_stream () const
  {
    return *yycdebug_;
  }

  void
  parser::set_debug_stream (std::ostream& o)
  {
    yycdebug_ = &o;
  }


  parser::debug_level_type
  parser::debug_level () const
  {
    return yydebug_;
  }

  void
  parser::set_debug_level (debug_level_type l)
  {
    yydebug_ = l;
  }
#endif // YYDEBUG

  parser::state_type
  parser::yy_lr_goto_state_ (state_type yystate, int yysym)
  {
    int yyr = yypgoto_[yysym - YYNTOKENS] + yystate;
    if (0 <= yyr && yyr <= yylast_ && yycheck_[yyr] == yystate)
      return yytable_[yyr];
    else
      return yydefgoto_[yysym - YYNTOKENS];
  }

  bool
  parser::yy_pact_value_is_default_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yypact_ninf_;
  }

  bool
  parser::yy_table_value_is_error_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yytable_ninf_;
  }

  int
  parser::operator() ()
  {
    return parse ();
  }

  int
  parser::parse ()
  {
    int yyn;
    /// Length of the RHS of the rule being reduced.
    int yylen = 0;

    // Error handling.
    int yynerrs_ = 0;
    int yyerrstatus_ = 0;

    /// The lookahead symbol.
    symbol_type yyla;

    /// The return value of parse ().
    int yyresult;

    // Discard the LAC context in case there still is one left from a
    // previous invocation.
    yy_lac_discard_ ("init");

#if YY_EXCEPTIONS
    try
#endif // YY_EXCEPTIONS
      {
    YYCDEBUG << "Starting parse\n";


    /* Initialize the stack.  The initial state will be set in
       yynewstate, since the latter expects the semantical and the
       location values to have been already stored, initialize these
       stacks with a primary value.  */
    yystack_.clear ();
    yypush_ (YY_NULLPTR, 0, YY_MOVE (yyla));

  /*-----------------------------------------------.
  | yynewstate -- push a new symbol on the stack.  |
  `-----------------------------------------------*/
  yynewstate:
    YYCDEBUG << "Entering state " << int (yystack_[0].state) << '\n';
    YY_STACK_PRINT ();

    // Accept?
    if (yystack_[0].state == yyfinal_)
      YYACCEPT;

    goto yybackup;


  /*-----------.
  | yybackup.  |
  `-----------*/
  yybackup:
    // Try to take a decision without lookahead.
    yyn = yypact_[+yystack_[0].state];
    if (yy_pact_value_is_default_ (yyn))
      goto yydefault;

    // Read a lookahead token.
    if (yyla.empty ())
      {
        YYCDEBUG << "Reading a token\n";
#if YY_EXCEPTIONS
        try
#endif // YY_EXCEPTIONS
          {
            symbol_type yylookahead (yylex ());
            yyla.move (yylookahead);
          }
#if YY_EXCEPTIONS
        catch (const syntax_error& yyexc)
          {
            YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
            error (yyexc);
            goto yyerrlab1;
          }
#endif // YY_EXCEPTIONS
      }
    YY_SYMBOL_PRINT ("Next token is", yyla);

    if (yyla.kind () == symbol_kind::S_YYerror)
    {
      // The scanner already issued an error message, process directly
      // to error recovery.  But do not keep the error token as
      // lookahead, it is too special and may lead us to an endless
      // loop in error recovery. */
      yyla.kind_ = symbol_kind::S_YYUNDEF;
      goto yyerrlab1;
    }

    /* If the proper action on seeing token YYLA.TYPE is to reduce or
       to detect an error, take that action.  */
    yyn += yyla.kind ();
    if (yyn < 0 || yylast_ < yyn || yycheck_[yyn] != yyla.kind ())
      {
        if (!yy_lac_establish_ (yyla.kind ()))
          goto yyerrlab;
        goto yydefault;
      }

    // Reduce or error.
    yyn = yytable_[yyn];
    if (yyn <= 0)
      {
        if (yy_table_value_is_error_ (yyn))
          goto yyerrlab;
        if (!yy_lac_establish_ (yyla.kind ()))
          goto yyerrlab;

        yyn = -yyn;
        goto yyreduce;
      }

    // Count tokens shifted since error; after three, turn off error status.
    if (yyerrstatus_)
      --yyerrstatus_;

    // Shift the lookahead token.
    yypush_ ("Shifting", state_type (yyn), YY_MOVE (yyla));
    yy_lac_discard_ ("shift");
    goto yynewstate;


  /*-----------------------------------------------------------.
  | yydefault -- do the default action for the current state.  |
  `-----------------------------------------------------------*/
  yydefault:
    yyn = yydefact_[+yystack_[0].state];
    if (yyn == 0)
      goto yyerrlab;
    goto yyreduce;


  /*-----------------------------.
  | yyreduce -- do a reduction.  |
  `-----------------------------*/
  yyreduce:
    yylen = yyr2_[yyn];
    {
      stack_symbol_type yylhs;
      yylhs.state = yy_lr_goto_state_ (yystack_[yylen].state, yyr1_[yyn]);
      /* Variants are always initialized to an empty instance of the
         correct type. The default '$$ = $1' action is NOT applied
         when using variants.  */
      switch (yyr1_[yyn])
    {
      case symbol_kind::S_align_spec: // align_spec
        yylhs.value.emplace< AlignmentSpec > ();
        break;

      case symbol_kind::S_assert_type: // assert_type
        yylhs.value.emplace< AssertionType > ();
        break;

      case symbol_kind::S_capture_rept: // capture_rept
      case symbol_kind::S_capture_macro: // capture_macro
        yylhs.value.emplace< Capture > ();
        break;

      case symbol_kind::S_reloc_8bit: // reloc_8bit
      case symbol_kind::S_reloc_8bit_no_str: // reloc_8bit_no_str
      case symbol_kind::S_reloc_8bit_offset: // reloc_8bit_offset
      case symbol_kind::S_reloc_16bit: // reloc_16bit
      case symbol_kind::S_reloc_16bit_no_str: // reloc_16bit_no_str
      case symbol_kind::S_relocexpr: // relocexpr
      case symbol_kind::S_relocexpr_no_str: // relocexpr_no_str
      case symbol_kind::S_op_mem_ind: // op_mem_ind
      case symbol_kind::S_op_a_n: // op_a_n
        yylhs.value.emplace< Expression > ();
        break;

      case symbol_kind::S_for_args: // for_args
        yylhs.value.emplace< ForArgs > ();
        break;

      case symbol_kind::S_compound_eq: // compound_eq
        yylhs.value.emplace< RPNCommand > ();
        break;

      case symbol_kind::S_sect_mod: // sect_mod
        yylhs.value.emplace< SectionModifier > ();
        break;

      case symbol_kind::S_sect_attrs: // sect_attrs
        yylhs.value.emplace< SectionSpec > ();
        break;

      case symbol_kind::S_strfmt_args: // strfmt_args
      case symbol_kind::S_strfmt_va_args: // strfmt_va_args
        yylhs.value.emplace< StrFmtArgList > ();
        break;

      case symbol_kind::S_NUMBER: // "number"
      case symbol_kind::S_shift_const: // shift_const
      case symbol_kind::S_rs_uconst: // rs_uconst
      case symbol_kind::S_bit_const: // bit_const
      case symbol_kind::S_uconst: // uconst
      case symbol_kind::S_const: // const
      case symbol_kind::S_const_no_str: // const_no_str
      case symbol_kind::S_const_8bit: // const_8bit
      case symbol_kind::S_opt_q_arg: // opt_q_arg
      case symbol_kind::S_sect_type: // sect_type
      case symbol_kind::S_sect_org: // sect_org
      case symbol_kind::S_op_a_r: // op_a_r
      case symbol_kind::S_ccode_expr: // ccode_expr
      case symbol_kind::S_ccode: // ccode
      case symbol_kind::S_reg_r: // reg_r
      case symbol_kind::S_reg_r_no_a: // reg_r_no_a
      case symbol_kind::S_reg_a: // reg_a
      case symbol_kind::S_reg_tt: // reg_tt
      case symbol_kind::S_reg_ss: // reg_ss
      case symbol_kind::S_reg_rr: // reg_rr
        yylhs.value.emplace< int32_t > ();
        break;

      case symbol_kind::S_macro_args: // macro_args
        yylhs.value.emplace< std::shared_ptr<MacroArgs> > ();
        break;

      case symbol_kind::S_STRING: // "string"
      case symbol_kind::S_LABEL: // "label"
      case symbol_kind::S_ID: // "identifier"
      case symbol_kind::S_LOCAL_ID: // "local identifier"
      case symbol_kind::S_ANON: // "anonymous label"
      case symbol_kind::S_def_id: // def_id
      case symbol_kind::S_redef_id: // redef_id
      case symbol_kind::S_scoped_id: // scoped_id
      case symbol_kind::S_scoped_anon_id: // scoped_anon_id
      case symbol_kind::S_string: // string
      case symbol_kind::S_strcat_args: // strcat_args
        yylhs.value.emplace< std::string > ();
        break;

      case symbol_kind::S_ds_args: // ds_args
        yylhs.value.emplace< std::vector<Expression> > ();
        break;

      case symbol_kind::S_purge_args: // purge_args
        yylhs.value.emplace< std::vector<std::string> > ();
        break;

      default:
        break;
    }



      // Perform the reduction.
      YY_REDUCE_PRINT (yyn);
#if YY_EXCEPTIONS
      try
#endif // YY_EXCEPTIONS
        {
          switch (yyn)
            {
  case 8: // opt_diff_mark: "+"
#line 323 "src/asm/parser.y"
                 {
		::error(
			"syntax error, unexpected + at the beginning of the line (is it a leftover diff mark?)\n"
		);
	}
#line 1049 "src/asm/parser.cpp"
    break;

  case 9: // opt_diff_mark: "-"
#line 328 "src/asm/parser.y"
                 {
		::error(
			"syntax error, unexpected - at the beginning of the line (is it a leftover diff mark?)\n"
		);
	}
#line 1059 "src/asm/parser.cpp"
    break;

  case 12: // $@1: %empty
#line 341 "src/asm/parser.y"
                {
		lexer_SetMode(LEXER_NORMAL);
		lexer_ToggleStringExpansion(true);
	}
#line 1068 "src/asm/parser.cpp"
    break;

  case 13: // line: error $@1 endofline
#line 344 "src/asm/parser.y"
                    {
		fstk_StopRept();
		yyerrok;
	}
#line 1077 "src/asm/parser.cpp"
    break;

  case 14: // $@2: %empty
#line 349 "src/asm/parser.y"
                      {
		lexer_SetMode(LEXER_NORMAL);
		lexer_ToggleStringExpansion(true);
	}
#line 1086 "src/asm/parser.cpp"
    break;

  case 15: // line: "label" error $@2 endofline
#line 352 "src/asm/parser.y"
                    {
		Symbol *macro = sym_FindExactSymbol(yystack_[3].value.as < std::string > ());

		if (macro && macro->type == SYM_MACRO)
			fprintf(
			    stderr,
			    "    To invoke `%s` as a macro it must be indented\n",
			    yystack_[3].value.as < std::string > ().c_str()
			);
		fstk_StopRept();
		yyerrok;
	}
#line 1103 "src/asm/parser.cpp"
    break;

  case 24: // if: "IF" const "newline"
#line 383 "src/asm/parser.y"
                             {
		lexer_IncIFDepth();

		if (yystack_[1].value.as < int32_t > ())
			lexer_RunIFBlock();
		else
			lexer_SetMode(LEXER_SKIP_TO_ELIF);
	}
#line 1116 "src/asm/parser.cpp"
    break;

  case 25: // elif: "ELIF" const "newline"
#line 394 "src/asm/parser.y"
                               {
		if (lexer_GetIFDepth() == 0)
			fatalerror("Found ELIF outside an IF construct\n");

		if (lexer_RanIFBlock()) {
			if (lexer_ReachedELSEBlock())
				fatalerror("Found ELIF after an ELSE block\n");

			lexer_SetMode(LEXER_SKIP_TO_ENDC);
		} else if (yystack_[1].value.as < int32_t > ()) {
			lexer_RunIFBlock();
		} else {
			lexer_SetMode(LEXER_SKIP_TO_ELIF);
		}
	}
#line 1136 "src/asm/parser.cpp"
    break;

  case 26: // else: "ELSE" "newline"
#line 412 "src/asm/parser.y"
                         {
		if (lexer_GetIFDepth() == 0)
			fatalerror("Found ELSE outside an IF construct\n");

		if (lexer_RanIFBlock()) {
			if (lexer_ReachedELSEBlock())
				fatalerror("Found ELSE after an ELSE block\n");

			lexer_SetMode(LEXER_SKIP_TO_ENDC);
		} else {
			lexer_RunIFBlock();
			lexer_ReachELSEBlock();
		}
	}
#line 1155 "src/asm/parser.cpp"
    break;

  case 31: // endc: "ENDC"
#line 438 "src/asm/parser.y"
                 {
		lexer_DecIFDepth();
	}
#line 1163 "src/asm/parser.cpp"
    break;

  case 32: // $@3: %empty
#line 444 "src/asm/parser.y"
               {
		lexer_ToggleStringExpansion(false);
	}
#line 1171 "src/asm/parser.cpp"
    break;

  case 33: // def_id: "DEF" $@3 "identifier"
#line 446 "src/asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1180 "src/asm/parser.cpp"
    break;

  case 34: // $@4: %empty
#line 453 "src/asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 1188 "src/asm/parser.cpp"
    break;

  case 35: // redef_id: "REDEF" $@4 "identifier"
#line 455 "src/asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1197 "src/asm/parser.cpp"
    break;

  case 36: // scoped_id: "identifier"
#line 464 "src/asm/parser.y"
           {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1205 "src/asm/parser.cpp"
    break;

  case 37: // scoped_id: "local identifier"
#line 467 "src/asm/parser.y"
                   {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1213 "src/asm/parser.cpp"
    break;

  case 38: // scoped_id: "label"
#line 470 "src/asm/parser.y"
                {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1221 "src/asm/parser.cpp"
    break;

  case 39: // scoped_anon_id: scoped_id
#line 476 "src/asm/parser.y"
                  {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1229 "src/asm/parser.cpp"
    break;

  case 40: // scoped_anon_id: "anonymous label"
#line 479 "src/asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 1237 "src/asm/parser.cpp"
    break;

  case 42: // label: ":"
#line 486 "src/asm/parser.y"
                {
		sym_AddAnonLabel();
	}
#line 1245 "src/asm/parser.cpp"
    break;

  case 43: // label: "local identifier"
#line 489 "src/asm/parser.y"
                   {
		sym_AddLocalLabel(yystack_[0].value.as < std::string > ());
	}
#line 1253 "src/asm/parser.cpp"
    break;

  case 44: // label: "local identifier" ":"
#line 492 "src/asm/parser.y"
                         {
		sym_AddLocalLabel(yystack_[1].value.as < std::string > ());
	}
#line 1261 "src/asm/parser.cpp"
    break;

  case 45: // label: "label" ":"
#line 495 "src/asm/parser.y"
                      {
		sym_AddLabel(yystack_[1].value.as < std::string > ());
	}
#line 1269 "src/asm/parser.cpp"
    break;

  case 46: // label: "local identifier" "::"
#line 498 "src/asm/parser.y"
                                {
		sym_AddLocalLabel(yystack_[1].value.as < std::string > ());
		sym_Export(yystack_[1].value.as < std::string > ());
	}
#line 1278 "src/asm/parser.cpp"
    break;

  case 47: // label: "label" "::"
#line 502 "src/asm/parser.y"
                             {
		sym_AddLabel(yystack_[1].value.as < std::string > ());
		sym_Export(yystack_[1].value.as < std::string > ());
	}
#line 1287 "src/asm/parser.cpp"
    break;

  case 48: // $@5: %empty
#line 509 "src/asm/parser.y"
           {
		// Parsing 'macroargs' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1296 "src/asm/parser.cpp"
    break;

  case 49: // macro: "identifier" $@5 macro_args
#line 512 "src/asm/parser.y"
                     {
		fstk_RunMacro(yystack_[2].value.as < std::string > (), yystack_[0].value.as < std::shared_ptr<MacroArgs> > ());
	}
#line 1304 "src/asm/parser.cpp"
    break;

  case 50: // macro_args: %empty
#line 518 "src/asm/parser.y"
               {
		yylhs.value.as < std::shared_ptr<MacroArgs> > () = std::make_shared<MacroArgs>();
	}
#line 1312 "src/asm/parser.cpp"
    break;

  case 51: // macro_args: macro_args "string"
#line 521 "src/asm/parser.y"
                            {
		yylhs.value.as < std::shared_ptr<MacroArgs> > () = std::move(yystack_[1].value.as < std::shared_ptr<MacroArgs> > ());
		yylhs.value.as < std::shared_ptr<MacroArgs> > ()->appendArg(std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 1321 "src/asm/parser.cpp"
    break;

  case 95: // compound_eq: "+="
#line 574 "src/asm/parser.y"
                  {
		yylhs.value.as < RPNCommand > () = RPN_ADD;
	}
#line 1329 "src/asm/parser.cpp"
    break;

  case 96: // compound_eq: "-="
#line 577 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SUB;
	}
#line 1337 "src/asm/parser.cpp"
    break;

  case 97: // compound_eq: "*="
#line 580 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_MUL;
	}
#line 1345 "src/asm/parser.cpp"
    break;

  case 98: // compound_eq: "/="
#line 583 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_DIV;
	}
#line 1353 "src/asm/parser.cpp"
    break;

  case 99: // compound_eq: "%="
#line 586 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_MOD;
	}
#line 1361 "src/asm/parser.cpp"
    break;

  case 100: // compound_eq: "^="
#line 589 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_XOR;
	}
#line 1369 "src/asm/parser.cpp"
    break;

  case 101: // compound_eq: "|="
#line 592 "src/asm/parser.y"
                   {
		yylhs.value.as < RPNCommand > () = RPN_OR;
	}
#line 1377 "src/asm/parser.cpp"
    break;

  case 102: // compound_eq: "&="
#line 595 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_AND;
	}
#line 1385 "src/asm/parser.cpp"
    break;

  case 103: // compound_eq: "<<="
#line 598 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SHL;
	}
#line 1393 "src/asm/parser.cpp"
    break;

  case 104: // compound_eq: ">>="
#line 601 "src/asm/parser.y"
                    {
		yylhs.value.as < RPNCommand > () = RPN_SHR;
	}
#line 1401 "src/asm/parser.cpp"
    break;

  case 105: // align: "ALIGN" align_spec
#line 607 "src/asm/parser.y"
                            {
		sect_AlignPC(yystack_[0].value.as < AlignmentSpec > ().alignment, yystack_[0].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1409 "src/asm/parser.cpp"
    break;

  case 106: // align_spec: uconst
#line 613 "src/asm/parser.y"
               {
		if (yystack_[0].value.as < int32_t > () > 16) {
			::error("Alignment must be between 0 and 16, not %u\n", yystack_[0].value.as < int32_t > ());
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else {
			yylhs.value.as < AlignmentSpec > ().alignment = yystack_[0].value.as < int32_t > ();
			yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		}
	}
#line 1423 "src/asm/parser.cpp"
    break;

  case 107: // align_spec: uconst "," const
#line 622 "src/asm/parser.y"
                             {
		if (yystack_[2].value.as < int32_t > () > 16) {
			::error("Alignment must be between 0 and 16, not %u\n", yystack_[2].value.as < int32_t > ());
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else if (yystack_[0].value.as < int32_t > () <= -(1 << yystack_[2].value.as < int32_t > ()) || yystack_[0].value.as < int32_t > () >= 1 << yystack_[2].value.as < int32_t > ()) {
			::error(
				"The absolute alignment offset (%" PRIu32 ") must be less than alignment size (%d)\n",
				(uint32_t)(yystack_[0].value.as < int32_t > () < 0 ? -yystack_[0].value.as < int32_t > () : yystack_[0].value.as < int32_t > ()),
				1 << yystack_[2].value.as < int32_t > ()
			);
			yylhs.value.as < AlignmentSpec > ().alignment = yylhs.value.as < AlignmentSpec > ().alignOfs = 0;
		} else {
			yylhs.value.as < AlignmentSpec > ().alignment = yystack_[2].value.as < int32_t > ();
			yylhs.value.as < AlignmentSpec > ().alignOfs = yystack_[0].value.as < int32_t > () < 0 ? (1 << yystack_[2].value.as < int32_t > ()) + yystack_[0].value.as < int32_t > () : yystack_[0].value.as < int32_t > ();
		}
	}
#line 1444 "src/asm/parser.cpp"
    break;

  case 108: // $@6: %empty
#line 641 "src/asm/parser.y"
                {
		// Parsing 'opt_list' will restore the lexer's normal mode
		lexer_SetMode(LEXER_RAW);
	}
#line 1453 "src/asm/parser.cpp"
    break;

  case 112: // opt_list_entry: "string"
#line 653 "src/asm/parser.y"
               {
		opt_Parse(yystack_[0].value.as < std::string > ().c_str());
	}
#line 1461 "src/asm/parser.cpp"
    break;

  case 113: // popo: "POPO"
#line 659 "src/asm/parser.y"
                 {
		opt_Pop();
	}
#line 1469 "src/asm/parser.cpp"
    break;

  case 114: // pusho: "PUSHO"
#line 665 "src/asm/parser.y"
                  {
		opt_Push();
	}
#line 1477 "src/asm/parser.cpp"
    break;

  case 115: // pops: "POPS"
#line 671 "src/asm/parser.y"
                 {
		sect_PopSection();
	}
#line 1485 "src/asm/parser.cpp"
    break;

  case 116: // pushs: "PUSHS"
#line 677 "src/asm/parser.y"
                  {
		sect_PushSection();
	}
#line 1493 "src/asm/parser.cpp"
    break;

  case 117: // endsection: "ENDSECTION"
#line 683 "src/asm/parser.y"
                       {
		sect_EndSection();
	}
#line 1501 "src/asm/parser.cpp"
    break;

  case 118: // fail: "FAIL" string
#line 689 "src/asm/parser.y"
                        {
		fatalerror("%s\n", yystack_[0].value.as < std::string > ().c_str());
	}
#line 1509 "src/asm/parser.cpp"
    break;

  case 119: // warn: "WARN" string
#line 695 "src/asm/parser.y"
                        {
		warning(WARNING_USER, "%s\n", yystack_[0].value.as < std::string > ().c_str());
	}
#line 1517 "src/asm/parser.cpp"
    break;

  case 120: // assert_type: %empty
#line 701 "src/asm/parser.y"
               {
		yylhs.value.as < AssertionType > () = ASSERT_ERROR;
	}
#line 1525 "src/asm/parser.cpp"
    break;

  case 121: // assert_type: "WARN" ","
#line 704 "src/asm/parser.y"
                         {
		yylhs.value.as < AssertionType > () = ASSERT_WARN;
	}
#line 1533 "src/asm/parser.cpp"
    break;

  case 122: // assert_type: "FAIL" ","
#line 707 "src/asm/parser.y"
                         {
		yylhs.value.as < AssertionType > () = ASSERT_ERROR;
	}
#line 1541 "src/asm/parser.cpp"
    break;

  case 123: // assert_type: "FATAL" ","
#line 710 "src/asm/parser.y"
                          {
		yylhs.value.as < AssertionType > () = ASSERT_FATAL;
	}
#line 1549 "src/asm/parser.cpp"
    break;

  case 124: // assert: "ASSERT" assert_type relocexpr
#line 716 "src/asm/parser.y"
                                         {
		if (!yystack_[0].value.as < Expression > ().isKnown()) {
			out_CreateAssert(yystack_[1].value.as < AssertionType > (), yystack_[0].value.as < Expression > (), "", sect_GetOutputOffset());
		} else if (yystack_[0].value.as < Expression > ().value() == 0) {
			failAssert(yystack_[1].value.as < AssertionType > ());
		}
	}
#line 1561 "src/asm/parser.cpp"
    break;

  case 125: // assert: "ASSERT" assert_type relocexpr "," string
#line 723 "src/asm/parser.y"
                                                        {
		if (!yystack_[2].value.as < Expression > ().isKnown()) {
			out_CreateAssert(yystack_[3].value.as < AssertionType > (), yystack_[2].value.as < Expression > (), yystack_[0].value.as < std::string > (), sect_GetOutputOffset());
		} else if (yystack_[2].value.as < Expression > ().value() == 0) {
			failAssertMsg(yystack_[3].value.as < AssertionType > (), yystack_[0].value.as < std::string > ());
		}
	}
#line 1573 "src/asm/parser.cpp"
    break;

  case 126: // assert: "STATIC_ASSERT" assert_type const
#line 730 "src/asm/parser.y"
                                              {
		if (yystack_[0].value.as < int32_t > () == 0)
			failAssert(yystack_[1].value.as < AssertionType > ());
	}
#line 1582 "src/asm/parser.cpp"
    break;

  case 127: // assert: "STATIC_ASSERT" assert_type const "," string
#line 734 "src/asm/parser.y"
                                                           {
		if (yystack_[2].value.as < int32_t > () == 0)
			failAssertMsg(yystack_[3].value.as < AssertionType > (), yystack_[0].value.as < std::string > ());
	}
#line 1591 "src/asm/parser.cpp"
    break;

  case 128: // shift: "SHIFT" shift_const
#line 741 "src/asm/parser.y"
                              {
		if (MacroArgs *macroArgs = fstk_GetCurrentMacroArgs(); macroArgs) {
			macroArgs->shiftArgs(yystack_[0].value.as < int32_t > ());
		} else {
			::error("Cannot shift macro arguments outside of a macro\n");
		}
	}
#line 1603 "src/asm/parser.cpp"
    break;

  case 129: // shift_const: %empty
#line 751 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = 1;
	}
#line 1611 "src/asm/parser.cpp"
    break;

  case 130: // shift_const: const
#line 754 "src/asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 1617 "src/asm/parser.cpp"
    break;

  case 131: // load: "LOAD" sect_mod string "," sect_type sect_org sect_attrs
#line 758 "src/asm/parser.y"
                                                                     {
		sect_SetLoadSection(yystack_[4].value.as < std::string > (), (SectionType)yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 1625 "src/asm/parser.cpp"
    break;

  case 132: // load: "ENDL"
#line 761 "src/asm/parser.y"
                   {
		sect_EndLoadSection();
	}
#line 1633 "src/asm/parser.cpp"
    break;

  case 133: // rept: "REPT" uconst "newline" capture_rept endofline
#line 767 "src/asm/parser.y"
                                                       {
		if (yystack_[1].value.as < Capture > ().span.ptr)
			fstk_RunRept(yystack_[3].value.as < int32_t > (), yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
	}
#line 1642 "src/asm/parser.cpp"
    break;

  case 134: // $@7: %empty
#line 774 "src/asm/parser.y"
                {
		lexer_ToggleStringExpansion(false);
	}
#line 1650 "src/asm/parser.cpp"
    break;

  case 135: // $@8: %empty
#line 776 "src/asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
	}
#line 1658 "src/asm/parser.cpp"
    break;

  case 136: // for: "FOR" $@7 "identifier" $@8 "," for_args "newline" capture_rept endofline
#line 778 "src/asm/parser.y"
                                                        {
		if (yystack_[1].value.as < Capture > ().span.ptr)
			fstk_RunFor(yystack_[6].value.as < std::string > (), yystack_[3].value.as < ForArgs > ().start, yystack_[3].value.as < ForArgs > ().stop, yystack_[3].value.as < ForArgs > ().step, yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
	}
#line 1667 "src/asm/parser.cpp"
    break;

  case 137: // capture_rept: %empty
#line 785 "src/asm/parser.y"
               {
		yylhs.value.as < Capture > () = lexer_CaptureRept();
	}
#line 1675 "src/asm/parser.cpp"
    break;

  case 138: // for_args: const
#line 791 "src/asm/parser.y"
              {
		yylhs.value.as < ForArgs > ().start = 0;
		yylhs.value.as < ForArgs > ().stop = yystack_[0].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = 1;
	}
#line 1685 "src/asm/parser.cpp"
    break;

  case 139: // for_args: const "," const
#line 796 "src/asm/parser.y"
                            {
		yylhs.value.as < ForArgs > ().start = yystack_[2].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().stop = yystack_[0].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = 1;
	}
#line 1695 "src/asm/parser.cpp"
    break;

  case 140: // for_args: const "," const "," const
#line 801 "src/asm/parser.y"
                                        {
		yylhs.value.as < ForArgs > ().start = yystack_[4].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().stop = yystack_[2].value.as < int32_t > ();
		yylhs.value.as < ForArgs > ().step = yystack_[0].value.as < int32_t > ();
	}
#line 1705 "src/asm/parser.cpp"
    break;

  case 141: // break: label "BREAK" endofline
#line 809 "src/asm/parser.y"
                                  {
		if (fstk_Break())
			lexer_SetMode(LEXER_SKIP_TO_ENDR);
	}
#line 1714 "src/asm/parser.cpp"
    break;

  case 142: // $@9: %empty
#line 816 "src/asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 1722 "src/asm/parser.cpp"
    break;

  case 143: // $@10: %empty
#line 818 "src/asm/parser.y"
             {
		lexer_ToggleStringExpansion(true);
	}
#line 1730 "src/asm/parser.cpp"
    break;

  case 144: // def_macro: "MACRO" $@9 "identifier" $@10 "newline" capture_macro endofline
#line 820 "src/asm/parser.y"
                                          {
		if (yystack_[1].value.as < Capture > ().span.ptr)
			sym_AddMacro(yystack_[4].value.as < std::string > (), yystack_[1].value.as < Capture > ().lineNo, yystack_[1].value.as < Capture > ().span);
	}
#line 1739 "src/asm/parser.cpp"
    break;

  case 145: // capture_macro: %empty
#line 827 "src/asm/parser.y"
               {
		yylhs.value.as < Capture > () = lexer_CaptureMacro();
	}
#line 1747 "src/asm/parser.cpp"
    break;

  case 146: // rsset: "RSSET" uconst
#line 833 "src/asm/parser.y"
                         {
		sym_SetRSValue(yystack_[0].value.as < int32_t > ());
	}
#line 1755 "src/asm/parser.cpp"
    break;

  case 147: // rsreset: "RSRESET"
#line 839 "src/asm/parser.y"
                    {
		sym_SetRSValue(0);
	}
#line 1763 "src/asm/parser.cpp"
    break;

  case 148: // rs_uconst: %empty
#line 845 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = 1;
	}
#line 1771 "src/asm/parser.cpp"
    break;

  case 149: // rs_uconst: uconst
#line 848 "src/asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 1777 "src/asm/parser.cpp"
    break;

  case 150: // union: "UNION"
#line 852 "src/asm/parser.y"
                  {
		sect_StartUnion();
	}
#line 1785 "src/asm/parser.cpp"
    break;

  case 151: // nextu: "NEXTU"
#line 858 "src/asm/parser.y"
                  {
		sect_NextUnionMember();
	}
#line 1793 "src/asm/parser.cpp"
    break;

  case 152: // endu: "ENDU"
#line 864 "src/asm/parser.y"
                 {
		sect_EndUnion();
	}
#line 1801 "src/asm/parser.cpp"
    break;

  case 153: // ds: "DS" uconst
#line 870 "src/asm/parser.y"
                      {
		sect_Skip(yystack_[0].value.as < int32_t > (), true);
	}
#line 1809 "src/asm/parser.cpp"
    break;

  case 154: // ds: "DS" uconst "," ds_args trailing_comma
#line 873 "src/asm/parser.y"
                                                     {
		sect_RelBytes(yystack_[3].value.as < int32_t > (), yystack_[1].value.as < std::vector<Expression> > ());
	}
#line 1817 "src/asm/parser.cpp"
    break;

  case 155: // ds: "DS" "ALIGN" "[" align_spec "]" trailing_comma
#line 876 "src/asm/parser.y"
                                                                  {
		uint32_t n = sect_GetAlignBytes(yystack_[2].value.as < AlignmentSpec > ().alignment, yystack_[2].value.as < AlignmentSpec > ().alignOfs);

		sect_Skip(n, true);
		sect_AlignPC(yystack_[2].value.as < AlignmentSpec > ().alignment, yystack_[2].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1828 "src/asm/parser.cpp"
    break;

  case 156: // ds: "DS" "ALIGN" "[" align_spec "]" "," ds_args trailing_comma
#line 882 "src/asm/parser.y"
                                                                                {
		uint32_t n = sect_GetAlignBytes(yystack_[4].value.as < AlignmentSpec > ().alignment, yystack_[4].value.as < AlignmentSpec > ().alignOfs);

		sect_RelBytes(n, yystack_[1].value.as < std::vector<Expression> > ());
		sect_AlignPC(yystack_[4].value.as < AlignmentSpec > ().alignment, yystack_[4].value.as < AlignmentSpec > ().alignOfs);
	}
#line 1839 "src/asm/parser.cpp"
    break;

  case 157: // ds_args: reloc_8bit
#line 891 "src/asm/parser.y"
                   {
		yylhs.value.as < std::vector<Expression> > ().push_back(std::move(yystack_[0].value.as < Expression > ()));
	}
#line 1847 "src/asm/parser.cpp"
    break;

  case 158: // ds_args: ds_args "," reloc_8bit
#line 894 "src/asm/parser.y"
                                   {
		yylhs.value.as < std::vector<Expression> > () = std::move(yystack_[2].value.as < std::vector<Expression> > ());
		yylhs.value.as < std::vector<Expression> > ().push_back(std::move(yystack_[0].value.as < Expression > ()));
	}
#line 1856 "src/asm/parser.cpp"
    break;

  case 159: // db: "DB"
#line 901 "src/asm/parser.y"
               {
		sect_Skip(1, false);
	}
#line 1864 "src/asm/parser.cpp"
    break;

  case 161: // dw: "DW"
#line 908 "src/asm/parser.y"
               {
		sect_Skip(2, false);
	}
#line 1872 "src/asm/parser.cpp"
    break;

  case 163: // dl: "DL"
#line 915 "src/asm/parser.y"
               {
		sect_Skip(4, false);
	}
#line 1880 "src/asm/parser.cpp"
    break;

  case 165: // def_equ: def_id "EQU" const
#line 922 "src/asm/parser.y"
                             {
		sym_AddEqu(yystack_[2].value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 1888 "src/asm/parser.cpp"
    break;

  case 166: // redef_equ: redef_id "EQU" const
#line 928 "src/asm/parser.y"
                               {
		sym_RedefEqu(yystack_[2].value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 1896 "src/asm/parser.cpp"
    break;

  case 167: // def_set: def_id "=" const
#line 934 "src/asm/parser.y"
                               {
		sym_AddVar(yystack_[2].value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 1904 "src/asm/parser.cpp"
    break;

  case 168: // def_set: redef_id "=" const
#line 937 "src/asm/parser.y"
                                   {
		sym_AddVar(yystack_[2].value.as < std::string > (), yystack_[0].value.as < int32_t > ());
	}
#line 1912 "src/asm/parser.cpp"
    break;

  case 169: // def_set: def_id compound_eq const
#line 940 "src/asm/parser.y"
                                   {
		compoundAssignment(yystack_[2].value.as < std::string > (), yystack_[1].value.as < RPNCommand > (), yystack_[0].value.as < int32_t > ());
	}
#line 1920 "src/asm/parser.cpp"
    break;

  case 170: // def_set: redef_id compound_eq const
#line 943 "src/asm/parser.y"
                                     {
		compoundAssignment(yystack_[2].value.as < std::string > (), yystack_[1].value.as < RPNCommand > (), yystack_[0].value.as < int32_t > ());
	}
#line 1928 "src/asm/parser.cpp"
    break;

  case 171: // def_rb: def_id "RB" rs_uconst
#line 949 "src/asm/parser.y"
                                {
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yystack_[2].value.as < std::string > (), rs);
		sym_SetRSValue(rs + yystack_[0].value.as < int32_t > ());
	}
#line 1938 "src/asm/parser.cpp"
    break;

  case 172: // def_rw: def_id "RW" rs_uconst
#line 957 "src/asm/parser.y"
                                {
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yystack_[2].value.as < std::string > (), rs);
		sym_SetRSValue(rs + 2 * yystack_[0].value.as < int32_t > ());
	}
#line 1948 "src/asm/parser.cpp"
    break;

  case 173: // def_rl: def_id "rl" rs_uconst
#line 965 "src/asm/parser.y"
                                {
		uint32_t rs = sym_GetRSValue();
		sym_AddEqu(yystack_[2].value.as < std::string > (), rs);
		sym_SetRSValue(rs + 4 * yystack_[0].value.as < int32_t > ());
	}
#line 1958 "src/asm/parser.cpp"
    break;

  case 174: // def_equs: def_id "EQUS" string
#line 973 "src/asm/parser.y"
                               {
		sym_AddString(yystack_[2].value.as < std::string > (), std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 1966 "src/asm/parser.cpp"
    break;

  case 175: // redef_equs: redef_id "EQUS" string
#line 979 "src/asm/parser.y"
                                 {
		sym_RedefString(yystack_[2].value.as < std::string > (), std::make_shared<std::string>(yystack_[0].value.as < std::string > ()));
	}
#line 1974 "src/asm/parser.cpp"
    break;

  case 176: // $@11: %empty
#line 985 "src/asm/parser.y"
                  {
		lexer_ToggleStringExpansion(false);
	}
#line 1982 "src/asm/parser.cpp"
    break;

  case 177: // purge: "PURGE" $@11 purge_args trailing_comma
#line 987 "src/asm/parser.y"
                                    {
		for (std::string &arg : yystack_[1].value.as < std::vector<std::string> > ())
			sym_Purge(arg);
		lexer_ToggleStringExpansion(true);
	}
#line 1992 "src/asm/parser.cpp"
    break;

  case 178: // purge_args: scoped_id
#line 995 "src/asm/parser.y"
                  {
		yylhs.value.as < std::vector<std::string> > ().push_back(yystack_[0].value.as < std::string > ());
	}
#line 2000 "src/asm/parser.cpp"
    break;

  case 179: // purge_args: purge_args "," scoped_id
#line 998 "src/asm/parser.y"
                                     {
		yylhs.value.as < std::vector<std::string> > () = std::move(yystack_[2].value.as < std::vector<std::string> > ());
		yylhs.value.as < std::vector<std::string> > ().push_back(yystack_[0].value.as < std::string > ());
	}
#line 2009 "src/asm/parser.cpp"
    break;

  case 183: // export_list_entry: scoped_id
#line 1012 "src/asm/parser.y"
                  {
		sym_Export(yystack_[0].value.as < std::string > ());
	}
#line 2017 "src/asm/parser.cpp"
    break;

  case 184: // include: label "INCLUDE" string endofline
#line 1018 "src/asm/parser.y"
                                           {
		fstk_RunInclude(yystack_[1].value.as < std::string > (), false);
		if (failedOnMissingInclude)
			YYACCEPT;
	}
#line 2027 "src/asm/parser.cpp"
    break;

  case 185: // incbin: "INCBIN" string
#line 1026 "src/asm/parser.y"
                          {
		sect_BinaryFile(yystack_[0].value.as < std::string > (), 0);
		if (failedOnMissingInclude)
			YYACCEPT;
	}
#line 2037 "src/asm/parser.cpp"
    break;

  case 186: // incbin: "INCBIN" string "," const
#line 1031 "src/asm/parser.y"
                                        {
		sect_BinaryFile(yystack_[2].value.as < std::string > (), yystack_[0].value.as < int32_t > ());
		if (failedOnMissingInclude)
			YYACCEPT;
	}
#line 2047 "src/asm/parser.cpp"
    break;

  case 187: // incbin: "INCBIN" string "," const "," const
#line 1036 "src/asm/parser.y"
                                                    {
		sect_BinaryFileSlice(yystack_[4].value.as < std::string > (), yystack_[2].value.as < int32_t > (), yystack_[0].value.as < int32_t > ());
		if (failedOnMissingInclude)
			YYACCEPT;
	}
#line 2057 "src/asm/parser.cpp"
    break;

  case 188: // charmap: "CHARMAP" string "," const_8bit
#line 1044 "src/asm/parser.y"
                                            {
		charmap_Add(yystack_[2].value.as < std::string > (), (uint8_t)yystack_[0].value.as < int32_t > ());
	}
#line 2065 "src/asm/parser.cpp"
    break;

  case 189: // newcharmap: "NEWCHARMAP" "identifier"
#line 1050 "src/asm/parser.y"
                          {
		charmap_New(yystack_[0].value.as < std::string > (), nullptr);
	}
#line 2073 "src/asm/parser.cpp"
    break;

  case 190: // newcharmap: "NEWCHARMAP" "identifier" "," "identifier"
#line 1053 "src/asm/parser.y"
                                     {
		charmap_New(yystack_[2].value.as < std::string > (), &yystack_[0].value.as < std::string > ());
	}
#line 2081 "src/asm/parser.cpp"
    break;

  case 191: // setcharmap: "SETCHARMAP" "identifier"
#line 1059 "src/asm/parser.y"
                          {
		charmap_Set(yystack_[0].value.as < std::string > ());
	}
#line 2089 "src/asm/parser.cpp"
    break;

  case 192: // pushc: "PUSHC"
#line 1065 "src/asm/parser.y"
                  {
		charmap_Push();
	}
#line 2097 "src/asm/parser.cpp"
    break;

  case 193: // popc: "POPC"
#line 1071 "src/asm/parser.y"
                 {
		charmap_Pop();
	}
#line 2105 "src/asm/parser.cpp"
    break;

  case 195: // println: "PRINTLN"
#line 1079 "src/asm/parser.y"
                    {
		putchar('\n');
		fflush(stdout);
	}
#line 2114 "src/asm/parser.cpp"
    break;

  case 196: // println: "PRINTLN" print_exprs trailing_comma
#line 1083 "src/asm/parser.y"
                                                 {
		putchar('\n');
		fflush(stdout);
	}
#line 2123 "src/asm/parser.cpp"
    break;

  case 199: // print_expr: const_no_str
#line 1095 "src/asm/parser.y"
                     {
		printf("$%" PRIX32, yystack_[0].value.as < int32_t > ());
	}
#line 2131 "src/asm/parser.cpp"
    break;

  case 200: // print_expr: string
#line 1098 "src/asm/parser.y"
                 {
		// Allow printing NUL characters
		fwrite(yystack_[0].value.as < std::string > ().data(), 1, yystack_[0].value.as < std::string > ().length(), stdout);
	}
#line 2140 "src/asm/parser.cpp"
    break;

  case 201: // bit_const: const
#line 1105 "src/asm/parser.y"
              {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0 || yylhs.value.as < int32_t > () > 7) {
			::error("Bit number must be between 0 and 7, not %" PRId32 "\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = 0;
		}
	}
#line 2152 "src/asm/parser.cpp"
    break;

  case 204: // constlist_8bit_entry: reloc_8bit_no_str
#line 1120 "src/asm/parser.y"
                          {
		sect_RelByte(yystack_[0].value.as < Expression > (), 0);
	}
#line 2160 "src/asm/parser.cpp"
    break;

  case 205: // constlist_8bit_entry: string
#line 1123 "src/asm/parser.y"
                 {
		std::vector<uint8_t> output;

		charmap_Convert(yystack_[0].value.as < std::string > (), output);
		sect_AbsByteGroup(output.data(), output.size());
	}
#line 2171 "src/asm/parser.cpp"
    break;

  case 208: // constlist_16bit_entry: reloc_16bit_no_str
#line 1137 "src/asm/parser.y"
                           {
		sect_RelWord(yystack_[0].value.as < Expression > (), 0);
	}
#line 2179 "src/asm/parser.cpp"
    break;

  case 209: // constlist_16bit_entry: string
#line 1140 "src/asm/parser.y"
                 {
		std::vector<uint8_t> output;

		charmap_Convert(yystack_[0].value.as < std::string > (), output);
		sect_AbsWordGroup(output.data(), output.size());
	}
#line 2190 "src/asm/parser.cpp"
    break;

  case 212: // constlist_32bit_entry: relocexpr_no_str
#line 1154 "src/asm/parser.y"
                         {
		sect_RelLong(yystack_[0].value.as < Expression > (), 0);
	}
#line 2198 "src/asm/parser.cpp"
    break;

  case 213: // constlist_32bit_entry: string
#line 1157 "src/asm/parser.y"
                 {
		std::vector<uint8_t> output;

		charmap_Convert(yystack_[0].value.as < std::string > (), output);
		sect_AbsLongGroup(output.data(), output.size());
	}
#line 2209 "src/asm/parser.cpp"
    break;

  case 214: // reloc_8bit: relocexpr
#line 1166 "src/asm/parser.y"
                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2218 "src/asm/parser.cpp"
    break;

  case 215: // reloc_8bit_no_str: relocexpr_no_str
#line 1173 "src/asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2227 "src/asm/parser.cpp"
    break;

  case 216: // reloc_8bit_offset: "+" relocexpr
#line 1180 "src/asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2236 "src/asm/parser.cpp"
    break;

  case 217: // reloc_8bit_offset: "-" relocexpr
#line 1184 "src/asm/parser.y"
                           {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().makeNeg();
		yylhs.value.as < Expression > ().checkNBit(8);
	}
#line 2246 "src/asm/parser.cpp"
    break;

  case 218: // reloc_16bit: relocexpr
#line 1192 "src/asm/parser.y"
                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(16);
	}
#line 2255 "src/asm/parser.cpp"
    break;

  case 219: // reloc_16bit_no_str: relocexpr_no_str
#line 1199 "src/asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().checkNBit(16);
	}
#line 2264 "src/asm/parser.cpp"
    break;

  case 220: // relocexpr: relocexpr_no_str
#line 1206 "src/asm/parser.y"
                         {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 2272 "src/asm/parser.cpp"
    break;

  case 221: // relocexpr: string
#line 1209 "src/asm/parser.y"
                 {
		std::vector<uint8_t> output;

		charmap_Convert(yystack_[0].value.as < std::string > (), output);
		yylhs.value.as < Expression > ().makeNumber(str2int2(output));
	}
#line 2283 "src/asm/parser.cpp"
    break;

  case 222: // relocexpr_no_str: scoped_anon_id
#line 1218 "src/asm/parser.y"
                       {
		yylhs.value.as < Expression > ().makeSymbol(yystack_[0].value.as < std::string > ());
	}
#line 2291 "src/asm/parser.cpp"
    break;

  case 223: // relocexpr_no_str: "number"
#line 1221 "src/asm/parser.y"
                 {
		yylhs.value.as < Expression > ().makeNumber(yystack_[0].value.as < int32_t > ());
	}
#line 2299 "src/asm/parser.cpp"
    break;

  case 224: // relocexpr_no_str: "!" relocexpr
#line 1224 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().makeLogicNot();
	}
#line 2308 "src/asm/parser.cpp"
    break;

  case 225: // relocexpr_no_str: relocexpr "||" relocexpr
#line 1228 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGOR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2316 "src/asm/parser.cpp"
    break;

  case 226: // relocexpr_no_str: relocexpr "&&" relocexpr
#line 1231 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGAND, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2324 "src/asm/parser.cpp"
    break;

  case 227: // relocexpr_no_str: relocexpr "==" relocexpr
#line 1234 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGEQ, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2332 "src/asm/parser.cpp"
    break;

  case 228: // relocexpr_no_str: relocexpr ">" relocexpr
#line 1237 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGGT, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2340 "src/asm/parser.cpp"
    break;

  case 229: // relocexpr_no_str: relocexpr "<" relocexpr
#line 1240 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGLT, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2348 "src/asm/parser.cpp"
    break;

  case 230: // relocexpr_no_str: relocexpr ">=" relocexpr
#line 1243 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGGE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2356 "src/asm/parser.cpp"
    break;

  case 231: // relocexpr_no_str: relocexpr "<=" relocexpr
#line 1246 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGLE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2364 "src/asm/parser.cpp"
    break;

  case 232: // relocexpr_no_str: relocexpr "!=" relocexpr
#line 1249 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_LOGNE, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2372 "src/asm/parser.cpp"
    break;

  case 233: // relocexpr_no_str: relocexpr "+" relocexpr
#line 1252 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_ADD, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2380 "src/asm/parser.cpp"
    break;

  case 234: // relocexpr_no_str: relocexpr "-" relocexpr
#line 1255 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SUB, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2388 "src/asm/parser.cpp"
    break;

  case 235: // relocexpr_no_str: relocexpr "^" relocexpr
#line 1258 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_XOR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2396 "src/asm/parser.cpp"
    break;

  case 236: // relocexpr_no_str: relocexpr "|" relocexpr
#line 1261 "src/asm/parser.y"
                                    {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_OR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2404 "src/asm/parser.cpp"
    break;

  case 237: // relocexpr_no_str: relocexpr "&" relocexpr
#line 1264 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_AND, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2412 "src/asm/parser.cpp"
    break;

  case 238: // relocexpr_no_str: relocexpr "<<" relocexpr
#line 1267 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SHL, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2420 "src/asm/parser.cpp"
    break;

  case 239: // relocexpr_no_str: relocexpr ">>" relocexpr
#line 1270 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_SHR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2428 "src/asm/parser.cpp"
    break;

  case 240: // relocexpr_no_str: relocexpr ">>>" relocexpr
#line 1273 "src/asm/parser.y"
                                      {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_USHR, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2436 "src/asm/parser.cpp"
    break;

  case 241: // relocexpr_no_str: relocexpr "*" relocexpr
#line 1276 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_MUL, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2444 "src/asm/parser.cpp"
    break;

  case 242: // relocexpr_no_str: relocexpr "/" relocexpr
#line 1279 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_DIV, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2452 "src/asm/parser.cpp"
    break;

  case 243: // relocexpr_no_str: relocexpr "%" relocexpr
#line 1282 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_MOD, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2460 "src/asm/parser.cpp"
    break;

  case 244: // relocexpr_no_str: relocexpr "**" relocexpr
#line 1285 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > ().makeBinaryOp(RPN_EXP, std::move(yystack_[2].value.as < Expression > ()), yystack_[0].value.as < Expression > ());
	}
#line 2468 "src/asm/parser.cpp"
    break;

  case 245: // relocexpr_no_str: "+" relocexpr
#line 1288 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 2476 "src/asm/parser.cpp"
    break;

  case 246: // relocexpr_no_str: "-" relocexpr
#line 1291 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().makeNeg();
	}
#line 2485 "src/asm/parser.cpp"
    break;

  case 247: // relocexpr_no_str: "~" relocexpr
#line 1295 "src/asm/parser.y"
                                     {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
		yylhs.value.as < Expression > ().makeNot();
	}
#line 2494 "src/asm/parser.cpp"
    break;

  case 248: // relocexpr_no_str: "HIGH" "(" relocexpr ")"
#line 1299 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
		yylhs.value.as < Expression > ().makeHigh();
	}
#line 2503 "src/asm/parser.cpp"
    break;

  case 249: // relocexpr_no_str: "LOW" "(" relocexpr ")"
#line 1303 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
		yylhs.value.as < Expression > ().makeLow();
	}
#line 2512 "src/asm/parser.cpp"
    break;

  case 250: // relocexpr_no_str: "ISCONST" "(" relocexpr ")"
#line 1307 "src/asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeNumber(yystack_[1].value.as < Expression > ().isKnown());
	}
#line 2520 "src/asm/parser.cpp"
    break;

  case 251: // relocexpr_no_str: "BANK" "(" scoped_anon_id ")"
#line 1310 "src/asm/parser.y"
                                               {
		// '@' is also an ID; it is handled here
		yylhs.value.as < Expression > ().makeBankSymbol(yystack_[1].value.as < std::string > ());
	}
#line 2529 "src/asm/parser.cpp"
    break;

  case 252: // relocexpr_no_str: "BANK" "(" string ")"
#line 1314 "src/asm/parser.y"
                                       {
		yylhs.value.as < Expression > ().makeBankSection(yystack_[1].value.as < std::string > ());
	}
#line 2537 "src/asm/parser.cpp"
    break;

  case 253: // relocexpr_no_str: "SIZEOF" "(" string ")"
#line 1317 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeSizeOfSection(yystack_[1].value.as < std::string > ());
	}
#line 2545 "src/asm/parser.cpp"
    break;

  case 254: // relocexpr_no_str: "STARTOF" "(" string ")"
#line 1320 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeStartOfSection(yystack_[1].value.as < std::string > ());
	}
#line 2553 "src/asm/parser.cpp"
    break;

  case 255: // relocexpr_no_str: "SIZEOF" "(" sect_type ")"
#line 1323 "src/asm/parser.y"
                                            {
		yylhs.value.as < Expression > ().makeSizeOfSectionType((SectionType)yystack_[1].value.as < int32_t > ());
	}
#line 2561 "src/asm/parser.cpp"
    break;

  case 256: // relocexpr_no_str: "STARTOF" "(" sect_type ")"
#line 1326 "src/asm/parser.y"
                                             {
		yylhs.value.as < Expression > ().makeStartOfSectionType((SectionType)yystack_[1].value.as < int32_t > ());
	}
#line 2569 "src/asm/parser.cpp"
    break;

  case 257: // $@12: %empty
#line 1329 "src/asm/parser.y"
                 {
		lexer_ToggleStringExpansion(false);
	}
#line 2577 "src/asm/parser.cpp"
    break;

  case 258: // relocexpr_no_str: "DEF" $@12 "(" scoped_anon_id ")"
#line 1331 "src/asm/parser.y"
                                       {
		yylhs.value.as < Expression > ().makeNumber(sym_FindScopedValidSymbol(yystack_[1].value.as < std::string > ()) != nullptr);
		lexer_ToggleStringExpansion(true);
	}
#line 2586 "src/asm/parser.cpp"
    break;

  case 259: // relocexpr_no_str: "ROUND" "(" const opt_q_arg ")"
#line 1335 "src/asm/parser.y"
                                                 {
		yylhs.value.as < Expression > ().makeNumber(fix_Round(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2594 "src/asm/parser.cpp"
    break;

  case 260: // relocexpr_no_str: "CEIL" "(" const opt_q_arg ")"
#line 1338 "src/asm/parser.y"
                                                {
		yylhs.value.as < Expression > ().makeNumber(fix_Ceil(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2602 "src/asm/parser.cpp"
    break;

  case 261: // relocexpr_no_str: "FLOOR" "(" const opt_q_arg ")"
#line 1341 "src/asm/parser.y"
                                                 {
		yylhs.value.as < Expression > ().makeNumber(fix_Floor(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2610 "src/asm/parser.cpp"
    break;

  case 262: // relocexpr_no_str: "FDIV" "(" const "," const opt_q_arg ")"
#line 1344 "src/asm/parser.y"
                                                            {
		yylhs.value.as < Expression > ().makeNumber(fix_Div(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2618 "src/asm/parser.cpp"
    break;

  case 263: // relocexpr_no_str: "FMUL" "(" const "," const opt_q_arg ")"
#line 1347 "src/asm/parser.y"
                                                            {
		yylhs.value.as < Expression > ().makeNumber(fix_Mul(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2626 "src/asm/parser.cpp"
    break;

  case 264: // relocexpr_no_str: "FMOD" "(" const "," const opt_q_arg ")"
#line 1350 "src/asm/parser.y"
                                                            {
		yylhs.value.as < Expression > ().makeNumber(fix_Mod(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2634 "src/asm/parser.cpp"
    break;

  case 265: // relocexpr_no_str: "POW" "(" const "," const opt_q_arg ")"
#line 1353 "src/asm/parser.y"
                                                           {
		yylhs.value.as < Expression > ().makeNumber(fix_Pow(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2642 "src/asm/parser.cpp"
    break;

  case 266: // relocexpr_no_str: "LOG" "(" const "," const opt_q_arg ")"
#line 1356 "src/asm/parser.y"
                                                           {
		yylhs.value.as < Expression > ().makeNumber(fix_Log(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2650 "src/asm/parser.cpp"
    break;

  case 267: // relocexpr_no_str: "SIN" "(" const opt_q_arg ")"
#line 1359 "src/asm/parser.y"
                                               {
		yylhs.value.as < Expression > ().makeNumber(fix_Sin(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2658 "src/asm/parser.cpp"
    break;

  case 268: // relocexpr_no_str: "COS" "(" const opt_q_arg ")"
#line 1362 "src/asm/parser.y"
                                               {
		yylhs.value.as < Expression > ().makeNumber(fix_Cos(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2666 "src/asm/parser.cpp"
    break;

  case 269: // relocexpr_no_str: "TAN" "(" const opt_q_arg ")"
#line 1365 "src/asm/parser.y"
                                               {
		yylhs.value.as < Expression > ().makeNumber(fix_Tan(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2674 "src/asm/parser.cpp"
    break;

  case 270: // relocexpr_no_str: "ASIN" "(" const opt_q_arg ")"
#line 1368 "src/asm/parser.y"
                                                {
		yylhs.value.as < Expression > ().makeNumber(fix_ASin(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2682 "src/asm/parser.cpp"
    break;

  case 271: // relocexpr_no_str: "ACOS" "(" const opt_q_arg ")"
#line 1371 "src/asm/parser.y"
                                                {
		yylhs.value.as < Expression > ().makeNumber(fix_ACos(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2690 "src/asm/parser.cpp"
    break;

  case 272: // relocexpr_no_str: "ATAN" "(" const opt_q_arg ")"
#line 1374 "src/asm/parser.y"
                                                {
		yylhs.value.as < Expression > ().makeNumber(fix_ATan(yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2698 "src/asm/parser.cpp"
    break;

  case 273: // relocexpr_no_str: "ATAN2" "(" const "," const opt_q_arg ")"
#line 1377 "src/asm/parser.y"
                                                             {
		yylhs.value.as < Expression > ().makeNumber(fix_ATan2(yystack_[4].value.as < int32_t > (), yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > ()));
	}
#line 2706 "src/asm/parser.cpp"
    break;

  case 274: // relocexpr_no_str: "STRCMP" "(" string "," string ")"
#line 1380 "src/asm/parser.y"
                                                      {
		yylhs.value.as < Expression > ().makeNumber(yystack_[3].value.as < std::string > ().compare(yystack_[1].value.as < std::string > ()));
	}
#line 2714 "src/asm/parser.cpp"
    break;

  case 275: // relocexpr_no_str: "STRIN" "(" string "," string ")"
#line 1383 "src/asm/parser.y"
                                                     {
		auto pos = yystack_[3].value.as < std::string > ().find(yystack_[1].value.as < std::string > ());

		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos + 1 : 0);
	}
#line 2724 "src/asm/parser.cpp"
    break;

  case 276: // relocexpr_no_str: "STRRIN" "(" string "," string ")"
#line 1388 "src/asm/parser.y"
                                                      {
		auto pos = yystack_[3].value.as < std::string > ().rfind(yystack_[1].value.as < std::string > ());

		yylhs.value.as < Expression > ().makeNumber(pos != std::string::npos ? pos + 1 : 0);
	}
#line 2734 "src/asm/parser.cpp"
    break;

  case 277: // relocexpr_no_str: "STRLEN" "(" string ")"
#line 1393 "src/asm/parser.y"
                                         {
		yylhs.value.as < Expression > ().makeNumber(strlenUTF8(yystack_[1].value.as < std::string > ()));
	}
#line 2742 "src/asm/parser.cpp"
    break;

  case 278: // relocexpr_no_str: "CHARLEN" "(" string ")"
#line 1396 "src/asm/parser.y"
                                          {
		yylhs.value.as < Expression > ().makeNumber(charlenUTF8(yystack_[1].value.as < std::string > ()));
	}
#line 2750 "src/asm/parser.cpp"
    break;

  case 279: // relocexpr_no_str: "INCHARMAP" "(" string ")"
#line 1399 "src/asm/parser.y"
                                            {
		yylhs.value.as < Expression > ().makeNumber(charmap_HasChar(yystack_[1].value.as < std::string > ()));
	}
#line 2758 "src/asm/parser.cpp"
    break;

  case 280: // relocexpr_no_str: "(" relocexpr ")"
#line 1402 "src/asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
	}
#line 2766 "src/asm/parser.cpp"
    break;

  case 281: // uconst: const
#line 1408 "src/asm/parser.y"
              {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0)
			fatalerror("Constant must not be negative: %d\n", yylhs.value.as < int32_t > ());
	}
#line 2776 "src/asm/parser.cpp"
    break;

  case 282: // const: relocexpr
#line 1416 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < Expression > ().getConstVal();
	}
#line 2784 "src/asm/parser.cpp"
    break;

  case 283: // const_no_str: relocexpr_no_str
#line 1422 "src/asm/parser.y"
                         {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < Expression > ().getConstVal();
	}
#line 2792 "src/asm/parser.cpp"
    break;

  case 284: // const_8bit: reloc_8bit
#line 1428 "src/asm/parser.y"
                   {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < Expression > ().getConstVal();
	}
#line 2800 "src/asm/parser.cpp"
    break;

  case 285: // opt_q_arg: %empty
#line 1434 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = fix_Precision();
	}
#line 2808 "src/asm/parser.cpp"
    break;

  case 286: // opt_q_arg: "," const
#line 1437 "src/asm/parser.y"
                      {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 1 || yylhs.value.as < int32_t > () > 31) {
			::error("Fixed-point precision must be between 1 and 31, not %" PRId32 "\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = fix_Precision();
		}
	}
#line 2820 "src/asm/parser.cpp"
    break;

  case 287: // string: "string"
#line 1447 "src/asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 2828 "src/asm/parser.cpp"
    break;

  case 288: // string: "STRSUB" "(" string "," const "," uconst ")"
#line 1450 "src/asm/parser.y"
                                                                  {
		size_t len = strlenUTF8(yystack_[5].value.as < std::string > ());
		uint32_t pos = adjustNegativePos(yystack_[3].value.as < int32_t > (), len, "STRSUB");

		yylhs.value.as < std::string > () = strsubUTF8(yystack_[5].value.as < std::string > (), pos, yystack_[1].value.as < int32_t > ());
	}
#line 2839 "src/asm/parser.cpp"
    break;

  case 289: // string: "STRSUB" "(" string "," const ")"
#line 1456 "src/asm/parser.y"
                                                     {
		size_t len = strlenUTF8(yystack_[3].value.as < std::string > ());
		uint32_t pos = adjustNegativePos(yystack_[1].value.as < int32_t > (), len, "STRSUB");

		yylhs.value.as < std::string > () = strsubUTF8(yystack_[3].value.as < std::string > (), pos, pos > len ? 0 : len + 1 - pos);
	}
#line 2850 "src/asm/parser.cpp"
    break;

  case 290: // string: "CHARSUB" "(" string "," const ")"
#line 1462 "src/asm/parser.y"
                                                      {
		size_t len = charlenUTF8(yystack_[3].value.as < std::string > ());
		uint32_t pos = adjustNegativePos(yystack_[1].value.as < int32_t > (), len, "CHARSUB");

		yylhs.value.as < std::string > () = charsubUTF8(yystack_[3].value.as < std::string > (), pos);
	}
#line 2861 "src/asm/parser.cpp"
    break;

  case 291: // string: "STRCAT" "(" ")"
#line 1468 "src/asm/parser.y"
                                  {
		yylhs.value.as < std::string > ().clear();
	}
#line 2869 "src/asm/parser.cpp"
    break;

  case 292: // string: "STRCAT" "(" strcat_args ")"
#line 1471 "src/asm/parser.y"
                                              {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
	}
#line 2877 "src/asm/parser.cpp"
    break;

  case 293: // string: "STRUPR" "(" string ")"
#line 1474 "src/asm/parser.y"
                                         {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
		std::transform(RANGE(yylhs.value.as < std::string > ()), yylhs.value.as < std::string > ().begin(), [](char c) { return toupper(c); });
	}
#line 2886 "src/asm/parser.cpp"
    break;

  case 294: // string: "STRLWR" "(" string ")"
#line 1478 "src/asm/parser.y"
                                         {
		yylhs.value.as < std::string > () = std::move(yystack_[1].value.as < std::string > ());
		std::transform(RANGE(yylhs.value.as < std::string > ()), yylhs.value.as < std::string > ().begin(), [](char c) { return tolower(c); });
	}
#line 2895 "src/asm/parser.cpp"
    break;

  case 295: // string: "STRRPL" "(" string "," string "," string ")"
#line 1482 "src/asm/parser.y"
                                                                   {
		yylhs.value.as < std::string > () = strrpl(yystack_[5].value.as < std::string > (), yystack_[3].value.as < std::string > (), yystack_[1].value.as < std::string > ());
	}
#line 2903 "src/asm/parser.cpp"
    break;

  case 296: // string: "STRFMT" "(" strfmt_args ")"
#line 1485 "src/asm/parser.y"
                                              {
		yylhs.value.as < std::string > () = strfmt(yystack_[1].value.as < StrFmtArgList > ().format, yystack_[1].value.as < StrFmtArgList > ().args);
	}
#line 2911 "src/asm/parser.cpp"
    break;

  case 297: // string: "SECTION" "(" scoped_anon_id ")"
#line 1488 "src/asm/parser.y"
                                                   {
		Symbol *sym = sym_FindScopedValidSymbol(yystack_[1].value.as < std::string > ());

		if (!sym)
			fatalerror("Unknown symbol \"%s\"\n", yystack_[1].value.as < std::string > ().c_str());
		Section const *section = sym->getSection();

		if (!section)
			fatalerror("\"%s\" does not belong to any section\n", sym->name.c_str());
		// Section names are capped by rgbasm's maximum string length,
		// so this currently can't overflow.
		yylhs.value.as < std::string > () = section->name;
	}
#line 2929 "src/asm/parser.cpp"
    break;

  case 298: // strcat_args: string
#line 1504 "src/asm/parser.y"
               {
		yylhs.value.as < std::string > () = std::move(yystack_[0].value.as < std::string > ());
	}
#line 2937 "src/asm/parser.cpp"
    break;

  case 299: // strcat_args: strcat_args "," string
#line 1507 "src/asm/parser.y"
                                   {
		yylhs.value.as < std::string > () = std::move(yystack_[2].value.as < std::string > ());
		yylhs.value.as < std::string > ().append(yystack_[0].value.as < std::string > ());
	}
#line 2946 "src/asm/parser.cpp"
    break;

  case 300: // strfmt_args: %empty
#line 1514 "src/asm/parser.y"
                 {}
#line 2952 "src/asm/parser.cpp"
    break;

  case 301: // strfmt_args: string strfmt_va_args
#line 1515 "src/asm/parser.y"
                                {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[0].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().format = std::move(yystack_[1].value.as < std::string > ());
	}
#line 2961 "src/asm/parser.cpp"
    break;

  case 302: // strfmt_va_args: %empty
#line 1522 "src/asm/parser.y"
                 {}
#line 2967 "src/asm/parser.cpp"
    break;

  case 303: // strfmt_va_args: strfmt_va_args "," const_no_str
#line 1523 "src/asm/parser.y"
                                            {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[2].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().args.push_back((uint32_t)yystack_[0].value.as < int32_t > ());
	}
#line 2976 "src/asm/parser.cpp"
    break;

  case 304: // strfmt_va_args: strfmt_va_args "," string
#line 1527 "src/asm/parser.y"
                                      {
		yylhs.value.as < StrFmtArgList > () = std::move(yystack_[2].value.as < StrFmtArgList > ());
		yylhs.value.as < StrFmtArgList > ().args.push_back(std::move(yystack_[0].value.as < std::string > ()));
	}
#line 2985 "src/asm/parser.cpp"
    break;

  case 305: // section: "SECTION" sect_mod string "," sect_type sect_org sect_attrs
#line 1534 "src/asm/parser.y"
                                                                        {
		sect_NewSection(yystack_[4].value.as < std::string > (), (SectionType)yystack_[2].value.as < int32_t > (), yystack_[1].value.as < int32_t > (), yystack_[0].value.as < SectionSpec > (), yystack_[5].value.as < SectionModifier > ());
	}
#line 2993 "src/asm/parser.cpp"
    break;

  case 306: // sect_mod: %empty
#line 1540 "src/asm/parser.y"
               {
		yylhs.value.as < SectionModifier > () = SECTION_NORMAL;
	}
#line 3001 "src/asm/parser.cpp"
    break;

  case 307: // sect_mod: "UNION"
#line 1543 "src/asm/parser.y"
                    {
		yylhs.value.as < SectionModifier > () = SECTION_UNION;
	}
#line 3009 "src/asm/parser.cpp"
    break;

  case 308: // sect_mod: "FRAGMENT"
#line 1546 "src/asm/parser.y"
                       {
		yylhs.value.as < SectionModifier > () = SECTION_FRAGMENT;
	}
#line 3017 "src/asm/parser.cpp"
    break;

  case 309: // sect_type: "WRAM0"
#line 1552 "src/asm/parser.y"
                   {
		yylhs.value.as < int32_t > () = SECTTYPE_WRAM0;
	}
#line 3025 "src/asm/parser.cpp"
    break;

  case 310: // sect_type: "VRAM"
#line 1555 "src/asm/parser.y"
                    {
		yylhs.value.as < int32_t > () = SECTTYPE_VRAM;
	}
#line 3033 "src/asm/parser.cpp"
    break;

  case 311: // sect_type: "ROMX"
#line 1558 "src/asm/parser.y"
                    {
		yylhs.value.as < int32_t > () = SECTTYPE_ROMX;
	}
#line 3041 "src/asm/parser.cpp"
    break;

  case 312: // sect_type: "ROM0"
#line 1561 "src/asm/parser.y"
                    {
		yylhs.value.as < int32_t > () = SECTTYPE_ROM0;
	}
#line 3049 "src/asm/parser.cpp"
    break;

  case 313: // sect_type: "HRAM"
#line 1564 "src/asm/parser.y"
                    {
		yylhs.value.as < int32_t > () = SECTTYPE_HRAM;
	}
#line 3057 "src/asm/parser.cpp"
    break;

  case 314: // sect_type: "WRAMX"
#line 1567 "src/asm/parser.y"
                     {
		yylhs.value.as < int32_t > () = SECTTYPE_WRAMX;
	}
#line 3065 "src/asm/parser.cpp"
    break;

  case 315: // sect_type: "SRAM"
#line 1570 "src/asm/parser.y"
                    {
		yylhs.value.as < int32_t > () = SECTTYPE_SRAM;
	}
#line 3073 "src/asm/parser.cpp"
    break;

  case 316: // sect_type: "OAM"
#line 1573 "src/asm/parser.y"
                   {
		yylhs.value.as < int32_t > () = SECTTYPE_OAM;
	}
#line 3081 "src/asm/parser.cpp"
    break;

  case 317: // sect_org: %empty
#line 1579 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = -1;
	}
#line 3089 "src/asm/parser.cpp"
    break;

  case 318: // sect_org: "[" uconst "]"
#line 1582 "src/asm/parser.y"
                               {
		yylhs.value.as < int32_t > () = yystack_[1].value.as < int32_t > ();
		if (yylhs.value.as < int32_t > () < 0 || yylhs.value.as < int32_t > () >= 0x10000) {
			::error("Address $%x is not 16-bit\n", yylhs.value.as < int32_t > ());
			yylhs.value.as < int32_t > () = -1;
		}
	}
#line 3101 "src/asm/parser.cpp"
    break;

  case 319: // sect_attrs: %empty
#line 1592 "src/asm/parser.y"
               {
		yylhs.value.as < SectionSpec > ().alignment = 0;
		yylhs.value.as < SectionSpec > ().alignOfs = 0;
		yylhs.value.as < SectionSpec > ().bank = -1;
	}
#line 3111 "src/asm/parser.cpp"
    break;

  case 320: // sect_attrs: sect_attrs "," "ALIGN" "[" align_spec "]"
#line 1597 "src/asm/parser.y"
                                                             {
		yylhs.value.as < SectionSpec > () = yystack_[5].value.as < SectionSpec > ();
		yylhs.value.as < SectionSpec > ().alignment = yystack_[1].value.as < AlignmentSpec > ().alignment;
		yylhs.value.as < SectionSpec > ().alignOfs = yystack_[1].value.as < AlignmentSpec > ().alignOfs;
	}
#line 3121 "src/asm/parser.cpp"
    break;

  case 321: // sect_attrs: sect_attrs "," "BANK" "[" uconst "]"
#line 1602 "src/asm/parser.y"
                                                        {
		yylhs.value.as < SectionSpec > () = yystack_[5].value.as < SectionSpec > ();
		yylhs.value.as < SectionSpec > ().bank = yystack_[1].value.as < int32_t > (); // We cannot check the validity of this yet
	}
#line 3130 "src/asm/parser.cpp"
    break;

  case 370: // z80_adc: "adc" op_a_n
#line 1665 "src/asm/parser.y"
                       {
		sect_AbsByte(0xCE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3139 "src/asm/parser.cpp"
    break;

  case 371: // z80_adc: "adc" op_a_r
#line 1669 "src/asm/parser.y"
                         {
		sect_AbsByte(0x88 | yystack_[0].value.as < int32_t > ());
	}
#line 3147 "src/asm/parser.cpp"
    break;

  case 372: // z80_add: "add" op_a_n
#line 1675 "src/asm/parser.y"
                       {
		sect_AbsByte(0xC6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3156 "src/asm/parser.cpp"
    break;

  case 373: // z80_add: "add" op_a_r
#line 1679 "src/asm/parser.y"
                         {
		sect_AbsByte(0x80 | yystack_[0].value.as < int32_t > ());
	}
#line 3164 "src/asm/parser.cpp"
    break;

  case 374: // z80_add: "add" "hl" "," reg_ss
#line 1682 "src/asm/parser.y"
                                       {
		sect_AbsByte(0x09 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3172 "src/asm/parser.cpp"
    break;

  case 375: // z80_add: "add" "sp" "," reloc_8bit
#line 1685 "src/asm/parser.y"
                                           {
		sect_AbsByte(0xE8);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3181 "src/asm/parser.cpp"
    break;

  case 376: // z80_and: "and" op_a_n
#line 1692 "src/asm/parser.y"
                       {
		sect_AbsByte(0xE6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3190 "src/asm/parser.cpp"
    break;

  case 377: // z80_and: "and" op_a_r
#line 1696 "src/asm/parser.y"
                         {
		sect_AbsByte(0xA0 | yystack_[0].value.as < int32_t > ());
	}
#line 3198 "src/asm/parser.cpp"
    break;

  case 378: // z80_bit: "bit" bit_const "," reg_r
#line 1702 "src/asm/parser.y"
                                      {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3207 "src/asm/parser.cpp"
    break;

  case 379: // z80_call: "call" reloc_16bit
#line 1709 "src/asm/parser.y"
                             {
		sect_AbsByte(0xCD);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3216 "src/asm/parser.cpp"
    break;

  case 380: // z80_call: "call" ccode_expr "," reloc_16bit
#line 1713 "src/asm/parser.y"
                                                {
		sect_AbsByte(0xC4 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3225 "src/asm/parser.cpp"
    break;

  case 381: // z80_ccf: "ccf"
#line 1720 "src/asm/parser.y"
                {
		sect_AbsByte(0x3F);
	}
#line 3233 "src/asm/parser.cpp"
    break;

  case 382: // z80_cp: "cp" op_a_n
#line 1726 "src/asm/parser.y"
                      {
		sect_AbsByte(0xFE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3242 "src/asm/parser.cpp"
    break;

  case 383: // z80_cp: "cp" op_a_r
#line 1730 "src/asm/parser.y"
                        {
		sect_AbsByte(0xB8 | yystack_[0].value.as < int32_t > ());
	}
#line 3250 "src/asm/parser.cpp"
    break;

  case 384: // z80_cpl: "cpl"
#line 1736 "src/asm/parser.y"
                {
		sect_AbsByte(0x2F);
	}
#line 3258 "src/asm/parser.cpp"
    break;

  case 385: // z80_daa: "daa"
#line 1742 "src/asm/parser.y"
                {
		sect_AbsByte(0x27);
	}
#line 3266 "src/asm/parser.cpp"
    break;

  case 386: // z80_dec: "dec" reg_r
#line 1748 "src/asm/parser.y"
                      {
		sect_AbsByte(0x05 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3274 "src/asm/parser.cpp"
    break;

  case 387: // z80_dec: "dec" reg_ss
#line 1751 "src/asm/parser.y"
                         {
		sect_AbsByte(0x0B | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3282 "src/asm/parser.cpp"
    break;

  case 388: // z80_di: "di"
#line 1757 "src/asm/parser.y"
               {
		sect_AbsByte(0xF3);
	}
#line 3290 "src/asm/parser.cpp"
    break;

  case 389: // z80_ei: "ei"
#line 1763 "src/asm/parser.y"
               {
		sect_AbsByte(0xFB);
	}
#line 3298 "src/asm/parser.cpp"
    break;

  case 390: // z80_halt: "halt"
#line 1769 "src/asm/parser.y"
                 {
		sect_AbsByte(0x76);
	}
#line 3306 "src/asm/parser.cpp"
    break;

  case 391: // z80_inc: "inc" reg_r
#line 1775 "src/asm/parser.y"
                      {
		sect_AbsByte(0x04 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3314 "src/asm/parser.cpp"
    break;

  case 392: // z80_inc: "inc" reg_ss
#line 1778 "src/asm/parser.y"
                         {
		sect_AbsByte(0x03 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3322 "src/asm/parser.cpp"
    break;

  case 393: // z80_jp: "jp" reloc_16bit
#line 1784 "src/asm/parser.y"
                           {
		sect_AbsByte(0xC3);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3331 "src/asm/parser.cpp"
    break;

  case 394: // z80_jp: "jp" ccode_expr "," reloc_16bit
#line 1788 "src/asm/parser.y"
                                              {
		sect_AbsByte(0xC2 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3340 "src/asm/parser.cpp"
    break;

  case 395: // z80_jp: "jp" "hl"
#line 1792 "src/asm/parser.y"
                         {
		sect_AbsByte(0xE9);
	}
#line 3348 "src/asm/parser.cpp"
    break;

  case 396: // z80_jr: "jr" reloc_16bit
#line 1798 "src/asm/parser.y"
                           {
		sect_AbsByte(0x18);
		sect_PCRelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3357 "src/asm/parser.cpp"
    break;

  case 397: // z80_jr: "jr" ccode_expr "," reloc_16bit
#line 1802 "src/asm/parser.y"
                                              {
		sect_AbsByte(0x20 | (yystack_[2].value.as < int32_t > () << 3));
		sect_PCRelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3366 "src/asm/parser.cpp"
    break;

  case 398: // z80_ldi: "ldi" "[" "hl" "]" "," MODE_A
#line 1809 "src/asm/parser.y"
                                                   {
		sect_AbsByte(0x02 | (2 << 4));
	}
#line 3374 "src/asm/parser.cpp"
    break;

  case 399: // z80_ldi: "ldi" MODE_A "," "[" "hl" "]"
#line 1812 "src/asm/parser.y"
                                                     {
		sect_AbsByte(0x0A | (2 << 4));
	}
#line 3382 "src/asm/parser.cpp"
    break;

  case 400: // z80_ldd: "ldd" "[" "hl" "]" "," MODE_A
#line 1818 "src/asm/parser.y"
                                                   {
		sect_AbsByte(0x02 | (3 << 4));
	}
#line 3390 "src/asm/parser.cpp"
    break;

  case 401: // z80_ldd: "ldd" MODE_A "," "[" "hl" "]"
#line 1821 "src/asm/parser.y"
                                                     {
		sect_AbsByte(0x0A | (3 << 4));
	}
#line 3398 "src/asm/parser.cpp"
    break;

  case 402: // z80_ldio: "ldh" MODE_A "," op_mem_ind
#line 1827 "src/asm/parser.y"
                                        {
		yystack_[0].value.as < Expression > ().makeCheckHRAM();

		sect_AbsByte(0xF0);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3409 "src/asm/parser.cpp"
    break;

  case 403: // z80_ldio: "ldh" op_mem_ind "," MODE_A
#line 1833 "src/asm/parser.y"
                                          {
		yystack_[2].value.as < Expression > ().makeCheckHRAM();

		sect_AbsByte(0xE0);
		sect_RelByte(yystack_[2].value.as < Expression > (), 1);
	}
#line 3420 "src/asm/parser.cpp"
    break;

  case 404: // z80_ldio: "ldh" MODE_A "," c_ind
#line 1839 "src/asm/parser.y"
                                     {
		sect_AbsByte(0xF2);
	}
#line 3428 "src/asm/parser.cpp"
    break;

  case 405: // z80_ldio: "ldh" c_ind "," MODE_A
#line 1842 "src/asm/parser.y"
                                     {
		sect_AbsByte(0xE2);
	}
#line 3436 "src/asm/parser.cpp"
    break;

  case 407: // c_ind: "[" relocexpr "+" MODE_C "]"
#line 1849 "src/asm/parser.y"
                                                {
		// This has to use `relocexpr`, not `const`, to avoid a shift/reduce conflict
		if (yystack_[3].value.as < Expression > ().getConstVal() != 0xFF00)
			::error("Base value must be equal to $FF00 for $FF00+C\n");
	}
#line 3446 "src/asm/parser.cpp"
    break;

  case 416: // z80_ld_hl: "ld" "hl" "," "sp" reloc_8bit_offset
#line 1868 "src/asm/parser.y"
                                                       {
		sect_AbsByte(0xF8);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3455 "src/asm/parser.cpp"
    break;

  case 417: // z80_ld_hl: "ld" "hl" "," reloc_16bit
#line 1872 "src/asm/parser.y"
                                           {
		sect_AbsByte(0x01 | (REG_HL << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3464 "src/asm/parser.cpp"
    break;

  case 418: // z80_ld_sp: "ld" "sp" "," "hl"
#line 1879 "src/asm/parser.y"
                                     {
		sect_AbsByte(0xF9);
	}
#line 3472 "src/asm/parser.cpp"
    break;

  case 419: // z80_ld_sp: "ld" "sp" "," reloc_16bit
#line 1882 "src/asm/parser.y"
                                           {
		sect_AbsByte(0x01 | (REG_SP << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3481 "src/asm/parser.cpp"
    break;

  case 420: // z80_ld_mem: "ld" op_mem_ind "," "sp"
#line 1889 "src/asm/parser.y"
                                        {
		sect_AbsByte(0x08);
		sect_RelWord(yystack_[2].value.as < Expression > (), 1);
	}
#line 3490 "src/asm/parser.cpp"
    break;

  case 421: // z80_ld_mem: "ld" op_mem_ind "," MODE_A
#line 1893 "src/asm/parser.y"
                                         {
		sect_AbsByte(0xEA);
		sect_RelWord(yystack_[2].value.as < Expression > (), 1);
	}
#line 3499 "src/asm/parser.cpp"
    break;

  case 422: // z80_ld_c_ind: "ld" c_ind "," MODE_A
#line 1900 "src/asm/parser.y"
                                  {
		sect_AbsByte(0xE2);
	}
#line 3507 "src/asm/parser.cpp"
    break;

  case 423: // z80_ld_rr: "ld" reg_rr "," MODE_A
#line 1906 "src/asm/parser.y"
                                   {
		sect_AbsByte(0x02 | (yystack_[2].value.as < int32_t > () << 4));
	}
#line 3515 "src/asm/parser.cpp"
    break;

  case 424: // z80_ld_r_no_a: "ld" reg_r_no_a "," reloc_8bit
#line 1912 "src/asm/parser.y"
                                           {
		sect_AbsByte(0x06 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3524 "src/asm/parser.cpp"
    break;

  case 425: // z80_ld_r_no_a: "ld" reg_r_no_a "," reg_r
#line 1916 "src/asm/parser.y"
                                        {
		if (yystack_[2].value.as < int32_t > () == REG_HL_IND && yystack_[0].value.as < int32_t > () == REG_HL_IND)
			::error("LD [HL], [HL] is not a valid instruction\n");
		else
			sect_AbsByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3535 "src/asm/parser.cpp"
    break;

  case 426: // z80_ld_a: "ld" reg_a "," reloc_8bit
#line 1925 "src/asm/parser.y"
                                      {
		sect_AbsByte(0x06 | (yystack_[2].value.as < int32_t > () << 3));
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3544 "src/asm/parser.cpp"
    break;

  case 427: // z80_ld_a: "ld" reg_a "," reg_r
#line 1929 "src/asm/parser.y"
                                   {
		sect_AbsByte(0x40 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3552 "src/asm/parser.cpp"
    break;

  case 428: // z80_ld_a: "ld" reg_a "," c_ind
#line 1932 "src/asm/parser.y"
                                   {
		sect_AbsByte(0xF2);
	}
#line 3560 "src/asm/parser.cpp"
    break;

  case 429: // z80_ld_a: "ld" reg_a "," reg_rr
#line 1935 "src/asm/parser.y"
                                    {
		sect_AbsByte(0x0A | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3568 "src/asm/parser.cpp"
    break;

  case 430: // z80_ld_a: "ld" reg_a "," op_mem_ind
#line 1938 "src/asm/parser.y"
                                        {
		sect_AbsByte(0xFA);
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3577 "src/asm/parser.cpp"
    break;

  case 431: // z80_ld_ss: "ld" "bc" "," reloc_16bit
#line 1945 "src/asm/parser.y"
                                         {
		sect_AbsByte(0x01 | (REG_BC << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3586 "src/asm/parser.cpp"
    break;

  case 432: // z80_ld_ss: "ld" "de" "," reloc_16bit
#line 1949 "src/asm/parser.y"
                                           {
		sect_AbsByte(0x01 | (REG_DE << 4));
		sect_RelWord(yystack_[0].value.as < Expression > (), 1);
	}
#line 3595 "src/asm/parser.cpp"
    break;

  case 433: // z80_nop: "nop"
#line 1958 "src/asm/parser.y"
                {
		sect_AbsByte(0x00);
	}
#line 3603 "src/asm/parser.cpp"
    break;

  case 434: // z80_or: "or" op_a_n
#line 1964 "src/asm/parser.y"
                      {
		sect_AbsByte(0xF6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3612 "src/asm/parser.cpp"
    break;

  case 435: // z80_or: "or" op_a_r
#line 1968 "src/asm/parser.y"
                        {
		sect_AbsByte(0xB0 | yystack_[0].value.as < int32_t > ());
	}
#line 3620 "src/asm/parser.cpp"
    break;

  case 436: // z80_pop: "pop" reg_tt
#line 1974 "src/asm/parser.y"
                       {
		sect_AbsByte(0xC1 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3628 "src/asm/parser.cpp"
    break;

  case 437: // z80_push: "push" reg_tt
#line 1980 "src/asm/parser.y"
                        {
		sect_AbsByte(0xC5 | (yystack_[0].value.as < int32_t > () << 4));
	}
#line 3636 "src/asm/parser.cpp"
    break;

  case 438: // z80_res: "res" bit_const "," reg_r
#line 1986 "src/asm/parser.y"
                                      {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x80 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3645 "src/asm/parser.cpp"
    break;

  case 439: // z80_ret: "ret"
#line 1993 "src/asm/parser.y"
                {
		sect_AbsByte(0xC9);
	}
#line 3653 "src/asm/parser.cpp"
    break;

  case 440: // z80_ret: "ret" ccode_expr
#line 1996 "src/asm/parser.y"
                             {
		sect_AbsByte(0xC0 | (yystack_[0].value.as < int32_t > () << 3));
	}
#line 3661 "src/asm/parser.cpp"
    break;

  case 441: // z80_reti: "reti"
#line 2002 "src/asm/parser.y"
                 {
		sect_AbsByte(0xD9);
	}
#line 3669 "src/asm/parser.cpp"
    break;

  case 442: // z80_rl: "rl" reg_r
#line 2008 "src/asm/parser.y"
                     {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x10 | yystack_[0].value.as < int32_t > ());
	}
#line 3678 "src/asm/parser.cpp"
    break;

  case 443: // z80_rla: "rla"
#line 2015 "src/asm/parser.y"
                {
		sect_AbsByte(0x17);
	}
#line 3686 "src/asm/parser.cpp"
    break;

  case 444: // z80_rlc: "rlc" reg_r
#line 2021 "src/asm/parser.y"
                      {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x00 | yystack_[0].value.as < int32_t > ());
	}
#line 3695 "src/asm/parser.cpp"
    break;

  case 445: // z80_rlca: "rlca"
#line 2028 "src/asm/parser.y"
                 {
		sect_AbsByte(0x07);
	}
#line 3703 "src/asm/parser.cpp"
    break;

  case 446: // z80_rr: "rr" reg_r
#line 2034 "src/asm/parser.y"
                     {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x18 | yystack_[0].value.as < int32_t > ());
	}
#line 3712 "src/asm/parser.cpp"
    break;

  case 447: // z80_rra: "rra"
#line 2041 "src/asm/parser.y"
                {
		sect_AbsByte(0x1F);
	}
#line 3720 "src/asm/parser.cpp"
    break;

  case 448: // z80_rrc: "rrc" reg_r
#line 2047 "src/asm/parser.y"
                      {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x08 | yystack_[0].value.as < int32_t > ());
	}
#line 3729 "src/asm/parser.cpp"
    break;

  case 449: // z80_rrca: "rrca"
#line 2054 "src/asm/parser.y"
                 {
		sect_AbsByte(0x0F);
	}
#line 3737 "src/asm/parser.cpp"
    break;

  case 450: // z80_rst: "rst" reloc_8bit
#line 2060 "src/asm/parser.y"
                           {
		yystack_[0].value.as < Expression > ().makeCheckRST();
		if (!yystack_[0].value.as < Expression > ().isKnown())
			sect_RelByte(yystack_[0].value.as < Expression > (), 0);
		else
			sect_AbsByte(0xC7 | yystack_[0].value.as < Expression > ().value());
	}
#line 3749 "src/asm/parser.cpp"
    break;

  case 451: // z80_sbc: "sbc" op_a_n
#line 2070 "src/asm/parser.y"
                       {
		sect_AbsByte(0xDE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3758 "src/asm/parser.cpp"
    break;

  case 452: // z80_sbc: "sbc" op_a_r
#line 2074 "src/asm/parser.y"
                         {
		sect_AbsByte(0x98 | yystack_[0].value.as < int32_t > ());
	}
#line 3766 "src/asm/parser.cpp"
    break;

  case 453: // z80_scf: "scf"
#line 2080 "src/asm/parser.y"
                {
		sect_AbsByte(0x37);
	}
#line 3774 "src/asm/parser.cpp"
    break;

  case 454: // z80_set: "set" bit_const "," reg_r
#line 2086 "src/asm/parser.y"
                                      {
		sect_AbsByte(0xCB);
		sect_AbsByte(0xC0 | (yystack_[2].value.as < int32_t > () << 3) | yystack_[0].value.as < int32_t > ());
	}
#line 3783 "src/asm/parser.cpp"
    break;

  case 455: // z80_sla: "sla" reg_r
#line 2093 "src/asm/parser.y"
                      {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x20 | yystack_[0].value.as < int32_t > ());
	}
#line 3792 "src/asm/parser.cpp"
    break;

  case 456: // z80_sra: "sra" reg_r
#line 2100 "src/asm/parser.y"
                      {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x28 | yystack_[0].value.as < int32_t > ());
	}
#line 3801 "src/asm/parser.cpp"
    break;

  case 457: // z80_srl: "srl" reg_r
#line 2107 "src/asm/parser.y"
                      {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x38 | yystack_[0].value.as < int32_t > ());
	}
#line 3810 "src/asm/parser.cpp"
    break;

  case 458: // z80_stop: "stop"
#line 2114 "src/asm/parser.y"
                 {
		sect_AbsByte(0x10);
		sect_AbsByte(0x00);
	}
#line 3819 "src/asm/parser.cpp"
    break;

  case 459: // z80_stop: "stop" reloc_8bit
#line 2118 "src/asm/parser.y"
                              {
		sect_AbsByte(0x10);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3828 "src/asm/parser.cpp"
    break;

  case 460: // z80_sub: "sub" op_a_n
#line 2125 "src/asm/parser.y"
                       {
		sect_AbsByte(0xD6);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3837 "src/asm/parser.cpp"
    break;

  case 461: // z80_sub: "sub" op_a_r
#line 2129 "src/asm/parser.y"
                         {
		sect_AbsByte(0x90 | yystack_[0].value.as < int32_t > ());
	}
#line 3845 "src/asm/parser.cpp"
    break;

  case 462: // z80_swap: "swap" reg_r
#line 2135 "src/asm/parser.y"
                       {
		sect_AbsByte(0xCB);
		sect_AbsByte(0x30 | yystack_[0].value.as < int32_t > ());
	}
#line 3854 "src/asm/parser.cpp"
    break;

  case 463: // z80_xor: "xor" op_a_n
#line 2142 "src/asm/parser.y"
                       {
		sect_AbsByte(0xEE);
		sect_RelByte(yystack_[0].value.as < Expression > (), 1);
	}
#line 3863 "src/asm/parser.cpp"
    break;

  case 464: // z80_xor: "xor" op_a_r
#line 2146 "src/asm/parser.y"
                         {
		sect_AbsByte(0xA8 | yystack_[0].value.as < int32_t > ());
	}
#line 3871 "src/asm/parser.cpp"
    break;

  case 465: // op_mem_ind: "[" reloc_16bit "]"
#line 2154 "src/asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[1].value.as < Expression > ());
	}
#line 3879 "src/asm/parser.cpp"
    break;

  case 466: // op_a_r: reg_r
#line 2160 "src/asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 3885 "src/asm/parser.cpp"
    break;

  case 467: // op_a_r: MODE_A "," reg_r
#line 2161 "src/asm/parser.y"
                             {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > ();
	}
#line 3893 "src/asm/parser.cpp"
    break;

  case 468: // op_a_n: reloc_8bit
#line 2167 "src/asm/parser.y"
                   {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 3901 "src/asm/parser.cpp"
    break;

  case 469: // op_a_n: MODE_A "," reloc_8bit
#line 2170 "src/asm/parser.y"
                                  {
		yylhs.value.as < Expression > () = std::move(yystack_[0].value.as < Expression > ());
	}
#line 3909 "src/asm/parser.cpp"
    break;

  case 484: // ccode_expr: ccode
#line 2213 "src/asm/parser.y"
          { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 3915 "src/asm/parser.cpp"
    break;

  case 485: // ccode_expr: "!" ccode_expr
#line 2214 "src/asm/parser.y"
                                 {
		yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > () ^ 1;
	}
#line 3923 "src/asm/parser.cpp"
    break;

  case 486: // ccode: "nz"
#line 2220 "src/asm/parser.y"
              {
		yylhs.value.as < int32_t > () = CC_NZ;
	}
#line 3931 "src/asm/parser.cpp"
    break;

  case 487: // ccode: "z"
#line 2223 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = CC_Z;
	}
#line 3939 "src/asm/parser.cpp"
    break;

  case 488: // ccode: "nc"
#line 2226 "src/asm/parser.y"
                {
		yylhs.value.as < int32_t > () = CC_NC;
	}
#line 3947 "src/asm/parser.cpp"
    break;

  case 489: // ccode: "c"
#line 2229 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = CC_C;
	}
#line 3955 "src/asm/parser.cpp"
    break;

  case 490: // reg_r: reg_r_no_a
#line 2234 "src/asm/parser.y"
       { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 3961 "src/asm/parser.cpp"
    break;

  case 491: // reg_r: reg_a
#line 2234 "src/asm/parser.y"
                    { yylhs.value.as < int32_t > () = yystack_[0].value.as < int32_t > (); }
#line 3967 "src/asm/parser.cpp"
    break;

  case 492: // reg_r_no_a: MODE_B
#line 2237 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = REG_B;
	}
#line 3975 "src/asm/parser.cpp"
    break;

  case 493: // reg_r_no_a: MODE_C
#line 2240 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_C;
	}
#line 3983 "src/asm/parser.cpp"
    break;

  case 494: // reg_r_no_a: MODE_D
#line 2243 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_D;
	}
#line 3991 "src/asm/parser.cpp"
    break;

  case 495: // reg_r_no_a: MODE_E
#line 2246 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_E;
	}
#line 3999 "src/asm/parser.cpp"
    break;

  case 496: // reg_r_no_a: MODE_H
#line 2249 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_H;
	}
#line 4007 "src/asm/parser.cpp"
    break;

  case 497: // reg_r_no_a: MODE_L
#line 2252 "src/asm/parser.y"
                 {
		yylhs.value.as < int32_t > () = REG_L;
	}
#line 4015 "src/asm/parser.cpp"
    break;

  case 498: // reg_r_no_a: "[" "hl" "]"
#line 2255 "src/asm/parser.y"
                                {
		yylhs.value.as < int32_t > () = REG_HL_IND;
	}
#line 4023 "src/asm/parser.cpp"
    break;

  case 499: // reg_a: MODE_A
#line 2261 "src/asm/parser.y"
               {
		yylhs.value.as < int32_t > () = REG_A;
	}
#line 4031 "src/asm/parser.cpp"
    break;

  case 500: // reg_tt: "bc"
#line 2267 "src/asm/parser.y"
                {
		yylhs.value.as < int32_t > () = REG_BC;
	}
#line 4039 "src/asm/parser.cpp"
    break;

  case 501: // reg_tt: "de"
#line 2270 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_DE;
	}
#line 4047 "src/asm/parser.cpp"
    break;

  case 502: // reg_tt: "hl"
#line 2273 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_HL;
	}
#line 4055 "src/asm/parser.cpp"
    break;

  case 503: // reg_tt: "af"
#line 2276 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_AF;
	}
#line 4063 "src/asm/parser.cpp"
    break;

  case 504: // reg_ss: "bc"
#line 2282 "src/asm/parser.y"
                {
		yylhs.value.as < int32_t > () = REG_BC;
	}
#line 4071 "src/asm/parser.cpp"
    break;

  case 505: // reg_ss: "de"
#line 2285 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_DE;
	}
#line 4079 "src/asm/parser.cpp"
    break;

  case 506: // reg_ss: "hl"
#line 2288 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_HL;
	}
#line 4087 "src/asm/parser.cpp"
    break;

  case 507: // reg_ss: "sp"
#line 2291 "src/asm/parser.y"
                  {
		yylhs.value.as < int32_t > () = REG_SP;
	}
#line 4095 "src/asm/parser.cpp"
    break;

  case 508: // reg_rr: "[" "bc" "]"
#line 2297 "src/asm/parser.y"
                              {
		yylhs.value.as < int32_t > () = REG_BC_IND;
	}
#line 4103 "src/asm/parser.cpp"
    break;

  case 509: // reg_rr: "[" "de" "]"
#line 2300 "src/asm/parser.y"
                                {
		yylhs.value.as < int32_t > () = REG_DE_IND;
	}
#line 4111 "src/asm/parser.cpp"
    break;

  case 510: // reg_rr: hl_ind_inc
#line 2303 "src/asm/parser.y"
                     {
		yylhs.value.as < int32_t > () = REG_HL_INDINC;
	}
#line 4119 "src/asm/parser.cpp"
    break;

  case 511: // reg_rr: hl_ind_dec
#line 2306 "src/asm/parser.y"
                     {
		yylhs.value.as < int32_t > () = REG_HL_INDDEC;
	}
#line 4127 "src/asm/parser.cpp"
    break;


#line 4131 "src/asm/parser.cpp"

            default:
              break;
            }
        }
#if YY_EXCEPTIONS
      catch (const syntax_error& yyexc)
        {
          YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
          error (yyexc);
          YYERROR;
        }
#endif // YY_EXCEPTIONS
      YY_SYMBOL_PRINT ("-> $$ =", yylhs);
      yypop_ (yylen);
      yylen = 0;

      // Shift the result of the reduction.
      yypush_ (YY_NULLPTR, YY_MOVE (yylhs));
    }
    goto yynewstate;


  /*--------------------------------------.
  | yyerrlab -- here on detecting error.  |
  `--------------------------------------*/
  yyerrlab:
    // If not already recovering from an error, report this error.
    if (!yyerrstatus_)
      {
        ++yynerrs_;
        context yyctx (*this, yyla);
        std::string msg = yysyntax_error_ (yyctx);
        error (YY_MOVE (msg));
      }


    if (yyerrstatus_ == 3)
      {
        /* If just tried and failed to reuse lookahead token after an
           error, discard it.  */

        // Return failure if at end of input.
        if (yyla.kind () == symbol_kind::S_YYEOF)
          YYABORT;
        else if (!yyla.empty ())
          {
            yy_destroy_ ("Error: discarding", yyla);
            yyla.clear ();
          }
      }

    // Else will try to reuse lookahead token after shifting the error token.
    goto yyerrlab1;


  /*---------------------------------------------------.
  | yyerrorlab -- error raised explicitly by YYERROR.  |
  `---------------------------------------------------*/
  yyerrorlab:
    /* Pacify compilers when the user code never invokes YYERROR and
       the label yyerrorlab therefore never appears in user code.  */
    if (false)
      YYERROR;

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYERROR.  */
    yypop_ (yylen);
    yylen = 0;
    YY_STACK_PRINT ();
    goto yyerrlab1;


  /*-------------------------------------------------------------.
  | yyerrlab1 -- common code for both syntax error and YYERROR.  |
  `-------------------------------------------------------------*/
  yyerrlab1:
    yyerrstatus_ = 3;   // Each real token shifted decrements this.
    // Pop stack until we find a state that shifts the error token.
    for (;;)
      {
        yyn = yypact_[+yystack_[0].state];
        if (!yy_pact_value_is_default_ (yyn))
          {
            yyn += symbol_kind::S_YYerror;
            if (0 <= yyn && yyn <= yylast_
                && yycheck_[yyn] == symbol_kind::S_YYerror)
              {
                yyn = yytable_[yyn];
                if (0 < yyn)
                  break;
              }
          }

        // Pop the current state because it cannot handle the error token.
        if (yystack_.size () == 1)
          YYABORT;

        yy_destroy_ ("Error: popping", yystack_[0]);
        yypop_ ();
        YY_STACK_PRINT ();
      }
    {
      stack_symbol_type error_token;


      // Shift the error token.
      yy_lac_discard_ ("error recovery");
      error_token.state = state_type (yyn);
      yypush_ ("Shifting", YY_MOVE (error_token));
    }
    goto yynewstate;


  /*-------------------------------------.
  | yyacceptlab -- YYACCEPT comes here.  |
  `-------------------------------------*/
  yyacceptlab:
    yyresult = 0;
    goto yyreturn;


  /*-----------------------------------.
  | yyabortlab -- YYABORT comes here.  |
  `-----------------------------------*/
  yyabortlab:
    yyresult = 1;
    goto yyreturn;


  /*-----------------------------------------------------.
  | yyreturn -- parsing is finished, return the result.  |
  `-----------------------------------------------------*/
  yyreturn:
    if (!yyla.empty ())
      yy_destroy_ ("Cleanup: discarding lookahead", yyla);

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYABORT or YYACCEPT.  */
    yypop_ (yylen);
    YY_STACK_PRINT ();
    while (1 < yystack_.size ())
      {
        yy_destroy_ ("Cleanup: popping", yystack_[0]);
        yypop_ ();
      }

    return yyresult;
  }
#if YY_EXCEPTIONS
    catch (...)
      {
        YYCDEBUG << "Exception caught: cleaning lookahead and stack\n";
        // Do not try to display the values of the reclaimed symbols,
        // as their printers might throw an exception.
        if (!yyla.empty ())
          yy_destroy_ (YY_NULLPTR, yyla);

        while (1 < yystack_.size ())
          {
            yy_destroy_ (YY_NULLPTR, yystack_[0]);
            yypop_ ();
          }
        throw;
      }
#endif // YY_EXCEPTIONS
  }

  void
  parser::error (const syntax_error& yyexc)
  {
    error (yyexc.what ());
  }

  const char *
  parser::symbol_name (symbol_kind_type yysymbol)
  {
    static const char *const yy_sname[] =
    {
    "end of file", "error", "invalid token", "number", "string", ".", ",",
  ":", "::", "[", "]", "(", ")", "newline", "!", "&&", "||", ">", "<",
  ">=", "<=", "!=", "==", "+", "-", "|", "^", "&", "<<", ">>", ">>>", "*",
  "/", "%", "~", "NEG", "**", "DEF", "BANK", "ALIGN", "SIZEOF", "STARTOF",
  "SIN", "COS", "TAN", "ASIN", "ACOS", "ATAN", "ATAN2", "FDIV", "FMUL",
  "FMOD", "POW", "LOG", "ROUND", "CEIL", "FLOOR", "HIGH", "LOW", "ISCONST",
  "STRCMP", "STRIN", "STRRIN", "STRSUB", "STRLEN", "STRCAT", "STRUPR",
  "STRLWR", "STRRPL", "STRFMT", "CHARLEN", "CHARSUB", "INCHARMAP", "label",
  "identifier", "local identifier", "anonymous label", "EQU", "=", "EQUS",
  "+=", "-=", "*=", "/=", "%=", "|=", "^=", "&=", "<<=", ">>=", "INCLUDE",
  "PRINT", "PRINTLN", "IF", "ELIF", "ELSE", "ENDC", "EXPORT", "DB", "DS",
  "DW", "DL", "SECTION", "FRAGMENT", "ENDSECTION", "RB", "RW", "MACRO",
  "ENDM", "RSRESET", "RSSET", "UNION", "NEXTU", "ENDU", "INCBIN", "REPT",
  "FOR", "CHARMAP", "NEWCHARMAP", "SETCHARMAP", "PUSHC", "POPC", "SHIFT",
  "ENDR", "BREAK", "LOAD", "ENDL", "FAIL", "WARN", "FATAL", "ASSERT",
  "STATIC_ASSERT", "PURGE", "REDEF", "POPS", "PUSHS", "POPO", "PUSHO",
  "OPT", "ROM0", "ROMX", "WRAM0", "WRAMX", "HRAM", "VRAM", "SRAM", "OAM",
  "adc", "add", "and", "bit", "call", "ccf", "cp", "cpl", "daa", "dec",
  "di", "ei", "halt", "inc", "jp", "jr", "ld", "ldi", "ldd", "ldh", "nop",
  "or", "pop", "push", "res", "ret", "reti", "rst", "rl", "rla", "rlc",
  "rlca", "rr", "rra", "rrc", "rrca", "sbc", "scf", "set", "stop", "sla",
  "sra", "srl", "sub", "swap", "xor", "a", "b", "c", "d", "e", "h", "l",
  "af", "bc", "de", "sp", "hl", "hld/hl-", "hli/hl+", "nz", "z", "nc",
  "end of buffer", "$accept", "asm_file", "lines", "endofline",
  "opt_diff_mark", "line", "$@1", "$@2", "line_directive", "if", "elif",
  "else", "plain_directive", "endc", "def_id", "$@3", "redef_id", "$@4",
  "scoped_id", "scoped_anon_id", "label", "macro", "$@5", "macro_args",
  "directive", "trailing_comma", "compound_eq", "align", "align_spec",
  "opt", "$@6", "opt_list", "opt_list_entry", "popo", "pusho", "pops",
  "pushs", "endsection", "fail", "warn", "assert_type", "assert", "shift",
  "shift_const", "load", "rept", "for", "$@7", "$@8", "capture_rept",
  "for_args", "break", "def_macro", "$@9", "$@10", "capture_macro",
  "rsset", "rsreset", "rs_uconst", "union", "nextu", "endu", "ds",
  "ds_args", "db", "dw", "dl", "def_equ", "redef_equ", "def_set", "def_rb",
  "def_rw", "def_rl", "def_equs", "redef_equs", "purge", "$@11",
  "purge_args", "export", "export_list", "export_list_entry", "include",
  "incbin", "charmap", "newcharmap", "setcharmap", "pushc", "popc",
  "print", "println", "print_exprs", "print_expr", "bit_const",
  "constlist_8bit", "constlist_8bit_entry", "constlist_16bit",
  "constlist_16bit_entry", "constlist_32bit", "constlist_32bit_entry",
  "reloc_8bit", "reloc_8bit_no_str", "reloc_8bit_offset", "reloc_16bit",
  "reloc_16bit_no_str", "relocexpr", "relocexpr_no_str", "$@12", "uconst",
  "const", "const_no_str", "const_8bit", "opt_q_arg", "string",
  "strcat_args", "strfmt_args", "strfmt_va_args", "section", "sect_mod",
  "sect_type", "sect_org", "sect_attrs", "cpu_commands", "cpu_command",
  "z80_adc", "z80_add", "z80_and", "z80_bit", "z80_call", "z80_ccf",
  "z80_cp", "z80_cpl", "z80_daa", "z80_dec", "z80_di", "z80_ei",
  "z80_halt", "z80_inc", "z80_jp", "z80_jr", "z80_ldi", "z80_ldd",
  "z80_ldio", "c_ind", "z80_ld", "z80_ld_hl", "z80_ld_sp", "z80_ld_mem",
  "z80_ld_c_ind", "z80_ld_rr", "z80_ld_r_no_a", "z80_ld_a", "z80_ld_ss",
  "z80_nop", "z80_or", "z80_pop", "z80_push", "z80_res", "z80_ret",
  "z80_reti", "z80_rl", "z80_rla", "z80_rlc", "z80_rlca", "z80_rr",
  "z80_rra", "z80_rrc", "z80_rrca", "z80_rst", "z80_sbc", "z80_scf",
  "z80_set", "z80_sla", "z80_sra", "z80_srl", "z80_stop", "z80_sub",
  "z80_swap", "z80_xor", "op_mem_ind", "op_a_r", "op_a_n", "MODE_A",
  "MODE_B", "MODE_C", "MODE_D", "MODE_E", "MODE_H", "MODE_L", "ccode_expr",
  "ccode", "reg_r", "reg_r_no_a", "reg_a", "reg_tt", "reg_ss", "reg_rr",
  "hl_ind_inc", "hl_ind_dec", YY_NULLPTR
    };
    return yy_sname[yysymbol];
  }



  // parser::context.
  parser::context::context (const parser& yyparser, const symbol_type& yyla)
    : yyparser_ (yyparser)
    , yyla_ (yyla)
  {}

  int
  parser::context::expected_tokens (symbol_kind_type yyarg[], int yyargn) const
  {
    // Actual number of expected tokens
    int yycount = 0;

#if YYDEBUG
    // Execute LAC once. We don't care if it is successful, we
    // only do it for the sake of debugging output.
    if (!yyparser_.yy_lac_established_)
      yyparser_.yy_lac_check_ (yyla_.kind ());
#endif

    for (int yyx = 0; yyx < YYNTOKENS; ++yyx)
      {
        symbol_kind_type yysym = YY_CAST (symbol_kind_type, yyx);
        if (yysym != symbol_kind::S_YYerror
            && yysym != symbol_kind::S_YYUNDEF
            && yyparser_.yy_lac_check_ (yysym))
          {
            if (!yyarg)
              ++yycount;
            else if (yycount == yyargn)
              return 0;
            else
              yyarg[yycount++] = yysym;
          }
      }
    if (yyarg && yycount == 0 && 0 < yyargn)
      yyarg[0] = symbol_kind::S_YYEMPTY;
    return yycount;
  }




  bool
  parser::yy_lac_check_ (symbol_kind_type yytoken) const
  {
    // Logically, the yylac_stack's lifetime is confined to this function.
    // Clear it, to get rid of potential left-overs from previous call.
    yylac_stack_.clear ();
    // Reduce until we encounter a shift and thereby accept the token.
#if YYDEBUG
    YYCDEBUG << "LAC: checking lookahead " << symbol_name (yytoken) << ':';
#endif
    std::ptrdiff_t lac_top = 0;
    while (true)
      {
        state_type top_state = (yylac_stack_.empty ()
                                ? yystack_[lac_top].state
                                : yylac_stack_.back ());
        int yyrule = yypact_[+top_state];
        if (yy_pact_value_is_default_ (yyrule)
            || (yyrule += yytoken) < 0 || yylast_ < yyrule
            || yycheck_[yyrule] != yytoken)
          {
            // Use the default action.
            yyrule = yydefact_[+top_state];
            if (yyrule == 0)
              {
                YYCDEBUG << " Err\n";
                return false;
              }
          }
        else
          {
            // Use the action from yytable.
            yyrule = yytable_[yyrule];
            if (yy_table_value_is_error_ (yyrule))
              {
                YYCDEBUG << " Err\n";
                return false;
              }
            if (0 < yyrule)
              {
                YYCDEBUG << " S" << yyrule << '\n';
                return true;
              }
            yyrule = -yyrule;
          }
        // By now we know we have to simulate a reduce.
        YYCDEBUG << " R" << yyrule - 1;
        // Pop the corresponding number of values from the stack.
        {
          std::ptrdiff_t yylen = yyr2_[yyrule];
          // First pop from the LAC stack as many tokens as possible.
          std::ptrdiff_t lac_size = std::ptrdiff_t (yylac_stack_.size ());
          if (yylen < lac_size)
            {
              yylac_stack_.resize (std::size_t (lac_size - yylen));
              yylen = 0;
            }
          else if (lac_size)
            {
              yylac_stack_.clear ();
              yylen -= lac_size;
            }
          // Only afterwards look at the main stack.
          // We simulate popping elements by incrementing lac_top.
          lac_top += yylen;
        }
        // Keep top_state in sync with the updated stack.
        top_state = (yylac_stack_.empty ()
                     ? yystack_[lac_top].state
                     : yylac_stack_.back ());
        // Push the resulting state of the reduction.
        state_type state = yy_lr_goto_state_ (top_state, yyr1_[yyrule]);
        YYCDEBUG << " G" << int (state);
        yylac_stack_.push_back (state);
      }
  }

  // Establish the initial context if no initial context currently exists.
  bool
  parser::yy_lac_establish_ (symbol_kind_type yytoken)
  {
    /* Establish the initial context for the current lookahead if no initial
       context is currently established.

       We define a context as a snapshot of the parser stacks.  We define
       the initial context for a lookahead as the context in which the
       parser initially examines that lookahead in order to select a
       syntactic action.  Thus, if the lookahead eventually proves
       syntactically unacceptable (possibly in a later context reached via a
       series of reductions), the initial context can be used to determine
       the exact set of tokens that would be syntactically acceptable in the
       lookahead's place.  Moreover, it is the context after which any
       further semantic actions would be erroneous because they would be
       determined by a syntactically unacceptable token.

       yy_lac_establish_ should be invoked when a reduction is about to be
       performed in an inconsistent state (which, for the purposes of LAC,
       includes consistent states that don't know they're consistent because
       their default reductions have been disabled).

       For parse.lac=full, the implementation of yy_lac_establish_ is as
       follows.  If no initial context is currently established for the
       current lookahead, then check if that lookahead can eventually be
       shifted if syntactic actions continue from the current context.  */
    if (yy_lac_established_)
      return true;
    else
      {
#if YYDEBUG
        YYCDEBUG << "LAC: initial context established for "
                 << symbol_name (yytoken) << '\n';
#endif
        yy_lac_established_ = true;
        return yy_lac_check_ (yytoken);
      }
  }

  // Discard any previous initial lookahead context.
  void
  parser::yy_lac_discard_ (const char* event)
  {
   /* Discard any previous initial lookahead context because of Event,
      which may be a lookahead change or an invalidation of the currently
      established initial context for the current lookahead.

      The most common example of a lookahead change is a shift.  An example
      of both cases is syntax error recovery.  That is, a syntax error
      occurs when the lookahead is syntactically erroneous for the
      currently established initial context, so error recovery manipulates
      the parser stacks to try to find a new initial context in which the
      current lookahead is syntactically acceptable.  If it fails to find
      such a context, it discards the lookahead.  */
    if (yy_lac_established_)
      {
        YYCDEBUG << "LAC: initial context discarded due to "
                 << event << '\n';
        yy_lac_established_ = false;
      }
  }


  int
  parser::yy_syntax_error_arguments_ (const context& yyctx,
                                                 symbol_kind_type yyarg[], int yyargn) const
  {
    /* There are many possibilities here to consider:
       - If this state is a consistent state with a default action, then
         the only way this function was invoked is if the default action
         is an error action.  In that case, don't check for expected
         tokens because there are none.
       - The only way there can be no lookahead present (in yyla) is
         if this state is a consistent state with a default action.
         Thus, detecting the absence of a lookahead is sufficient to
         determine that there is no unexpected or expected token to
         report.  In that case, just report a simple "syntax error".
       - Don't assume there isn't a lookahead just because this state is
         a consistent state with a default action.  There might have
         been a previous inconsistent state, consistent state with a
         non-default action, or user semantic action that manipulated
         yyla.  (However, yyla is currently not documented for users.)
         In the first two cases, it might appear that the current syntax
         error should have been detected in the previous state when
         yy_lac_check was invoked.  However, at that time, there might
         have been a different syntax error that discarded a different
         initial context during error recovery, leaving behind the
         current lookahead.
    */

    if (!yyctx.lookahead ().empty ())
      {
        if (yyarg)
          yyarg[0] = yyctx.token ();
        int yyn = yyctx.expected_tokens (yyarg ? yyarg + 1 : yyarg, yyargn - 1);
        return yyn + 1;
      }
    return 0;
  }

  // Generate an error message.
  std::string
  parser::yysyntax_error_ (const context& yyctx) const
  {
    // Its maximum.
    enum { YYARGS_MAX = 5 };
    // Arguments of yyformat.
    symbol_kind_type yyarg[YYARGS_MAX];
    int yycount = yy_syntax_error_arguments_ (yyctx, yyarg, YYARGS_MAX);

    char const* yyformat = YY_NULLPTR;
    switch (yycount)
      {
#define YYCASE_(N, S)                         \
        case N:                               \
          yyformat = S;                       \
        break
      default: // Avoid compiler warnings.
        YYCASE_ (0, YY_("syntax error"));
        YYCASE_ (1, YY_("syntax error, unexpected %s"));
        YYCASE_ (2, YY_("syntax error, unexpected %s, expecting %s"));
        YYCASE_ (3, YY_("syntax error, unexpected %s, expecting %s or %s"));
        YYCASE_ (4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
        YYCASE_ (5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
#undef YYCASE_
      }

    std::string yyres;
    // Argument number.
    std::ptrdiff_t yyi = 0;
    for (char const* yyp = yyformat; *yyp; ++yyp)
      if (yyp[0] == '%' && yyp[1] == 's' && yyi < yycount)
        {
          yyres += symbol_name (yyarg[yyi++]);
          ++yyp;
        }
      else
        yyres += *yyp;
    return yyres;
  }


  const short parser::yypact_ninf_ = -605;

  const short parser::yytable_ninf_ = -305;

  const short
  parser::yypact_[] =
  {
    -605,    34,   110,  -605,  -605,  -605,   867,  -605,  -605,    82,
     114,  2300,  2300,    96,  -605,  2300,  -605,  -605,  -605,  -605,
    -605,  -605,    -1,  2036,  -605,  -605,  -605,  -605,  -605,    -1,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  2300,  2300,  2300,
    2300,  2300,  -605,   106,   139,   145,   157,   166,   167,   169,
     170,   172,   173,   179,   181,   182,   189,   201,   203,   232,
     234,   235,   248,   254,   263,   264,   269,   270,   272,   273,
     289,   290,   298,   303,   304,   315,   316,  -605,  -605,  -605,
    -605,   317,  -605,  -605,  2362,  -605,   223,  -605,   319,  -605,
     255,   340,  -605,   256,  -605,  -605,  -605,  -605,  2300,  -605,
      60,  2300,  2300,  -605,    26,  2300,  2226,  2300,  2300,   -82,
    -605,  -605,  2300,  -605,  -605,  -605,    60,    60,   280,   281,
    -605,  -605,  2300,    -1,   -82,  -605,    60,    60,   142,   142,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  1820,  1307,  1820,
    2300,  1131,  -605,  1820,  -605,  -605,    29,  -605,  -605,  -605,
      29,  1057,  1131,   109,    49,    75,    79,  -605,  1820,     3,
       3,  2300,    40,  -605,  2300,   180,  -605,   180,  -605,   180,
    -605,   180,  -605,  1820,  -605,  2300,  2300,   180,   180,   180,
    1820,   180,  1820,  -605,   343,   866,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,   348,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,    -1,   779,   321,   321,   321,
     321,   349,   573,   619,   619,  2300,  2300,  2300,  2300,  2300,
    2300,  2300,  2300,  2300,  2300,  2300,  2300,  2300,  2300,  2300,
    2300,  2300,  2300,    60,    60,    60,    60,    60,   390,    60,
      60,    60,    60,    60,    60,    60,   143,  2300,  2300,  2300,
    2300,  2300,  2300,  2300,  2300,  2300,  2300,  2300,  2300,  2300,
    2300,  2300,  2300,  2300,  2300,  2300,  2300,  -605,  -605,  -605,
    -605,  -605,   285,  -605,   355,  -605,    -1,   356,  -605,  2362,
      19,  -605,    24,   356,  -605,   363,  -605,   364,  -605,  -605,
      57,    66,   362,   366,   374,  -605,  -605,    67,    68,   375,
    -605,    69,    72,  -605,  -605,    60,  -605,   376,   377,   378,
    -605,  -605,  -605,  -605,    60,  -605,  -605,   379,   383,   384,
    2300,  2300,    26,   318,   389,   197,   393,   394,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  2362,  -605,  -605,   427,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,   438,
     439,  -605,  -605,  -605,  -605,   440,  -605,  1131,  -605,  -605,
    -605,  -605,  -605,  2362,   441,  -605,  -605,  -605,   443,   449,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,   444,  -605,   445,  1233,   446,   456,   457,   458,   459,
     461,   462,   463,   465,  -605,  -605,   271,   467,   468,   276,
     470,  1987,   475,   476,   477,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,   478,    40,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,   479,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  2300,  2300,    60,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  2300,  2300,  2300,  2300,
    2300,  2300,    60,  2300,  2256,  -605,  -605,   143,   474,   481,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,   482,   483,
     484,   485,   492,   492,   492,   492,   492,   492,   493,   494,
     496,   499,   500,   501,   492,   492,   492,   898,  1196,  1372,
     502,   503,   504,   508,   505,  -605,  -605,   129,   512,   513,
     520,  -605,   516,   517,   526,   530,   533,  1545,  2454,   527,
     527,   527,   527,   527,   527,   225,   225,   367,   367,   367,
     208,   208,   208,   321,   321,   321,  -605,   536,    -1,   540,
    -605,  2300,   560,  -605,  2300,  -605,  -605,    26,  -605,  2300,
    -605,  2300,  2300,  2300,  -605,  2300,  -605,   559,  2300,  2300,
     415,   561,  -605,  -605,  -605,   751,   562,  -605,   563,  -605,
    -605,   389,  -605,   556,  1407,  1484,  1820,  2300,   118,   180,
    -605,  2300,    62,   -49,  2300,  2300,   564,   568,   571,    92,
     572,   574,   575,  2434,   576,  2300,  2300,  1576,  1653,   -24,
     -18,  1820,  1907,   -24,   577,   370,   565,   578,   580,   -24,
     -24,   585,   180,   180,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,   583,  -605,  -605,
    -605,  -605,  -605,  -605,  2300,   584,   587,   588,   590,   591,
     592,  2300,  2300,  2300,  2300,  2300,  2300,   593,   594,   595,
    -605,  -605,  -605,    60,    60,    60,  2300,  -605,    60,  -605,
    -605,  -605,    60,   602,  -605,  -605,  2300,  -605,  -605,  -605,
    -605,  2300,  -605,  -605,  -605,  -605,  -605,   600,   605,  -605,
    -605,  -605,   296,   606,  -605,  -605,  -605,   296,    60,    60,
      26,  -605,  -605,  -605,   601,   603,   604,   607,   609,   610,
     612,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  1745,
    -605,  -605,   617,   620,  -605,  -605,  -605,  1987,  -605,  -605,
    -605,  -605,  -605,   150,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,   608,   413,   626,   429,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,   492,   492,   492,   492,   492,   492,
    -605,  -605,  -605,   622,   625,   631,   134,  -605,   639,  2300,
     642,    -1,   558,   649,   650,  2300,  -605,   653,  2300,   653,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,   648,  2300,  2300,  -605,   -24,   654,   -24,   655,
     647,   651,   656,   657,   658,   664,  -605,  -605,  -605,  2300,
    -605,    60,  -605,   136,  -605,  -605,  -605,  2300,  2300,  -605,
    -605,  2300,  -605,  -605,  -605,  -605,  2362,  2362,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,   667,   668,
      -1,   660,   605,   671,   661,   661,  -605,  -605,  -605,  2300,
    -605,  -605,    99,  -605,   682,   683,  2300,  2300,   685,   686,
    -605,  -605
  };

  const short
  parser::yydefact_[] =
  {
       3,     0,     7,     1,     8,     9,     0,    12,    42,     0,
      43,     0,     0,     0,   142,     0,   134,     4,    11,    21,
      22,    23,     0,    27,    17,    18,    19,    16,    20,     0,
      14,    45,    47,    44,    46,   223,   287,     0,     0,     0,
       0,     0,   257,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    38,    36,    37,
      40,     0,    39,   222,   282,   220,     0,   221,     0,    26,
       0,     0,   281,     0,     5,     6,    10,    32,     0,    48,
       0,     0,   195,    31,     0,   159,     0,   161,   163,   306,
     117,   147,     0,   150,   151,   152,     0,     0,     0,     0,
     192,   193,   129,     0,   306,   132,     0,     0,   120,   120,
     176,    34,   115,   116,   113,   114,   108,     0,     0,     0,
       0,     0,   381,     0,   384,   385,     0,   388,   389,   390,
       0,     0,     0,     0,     0,     0,     0,   433,     0,     0,
       0,     0,   439,   441,     0,     0,   443,     0,   445,     0,
     447,     0,   449,     0,   453,     0,   458,     0,     0,     0,
       0,     0,     0,    52,     0,     0,    29,    30,    92,    91,
      89,    90,    86,    87,    88,    74,    75,    76,    73,    72,
      62,    61,    63,    64,    65,    59,    56,    57,    58,    77,
      78,    79,    80,    81,    82,    83,    84,    85,    55,    66,
      67,    68,    69,    70,    71,    53,    54,    60,    28,   322,
     324,   325,   326,   327,   328,   329,   330,   331,   332,   333,
     334,   335,   336,   337,   338,   339,   342,   341,   343,   340,
     412,   413,   408,   409,   410,   414,   415,   411,   344,   345,
     346,   347,   348,   349,   350,   351,   352,   353,   354,   355,
     356,   357,   358,   359,   360,   361,   362,   363,   364,   365,
     366,   367,   368,   369,    13,     0,     0,   224,   245,   246,
     247,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,   300,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    24,    25,   143,
     137,   135,     0,   105,   106,    50,     0,    93,   197,     0,
     220,   199,   221,    93,   183,    93,   181,    93,   202,   204,
     220,   221,     0,   153,    93,   206,   208,   220,   221,    93,
     210,   220,   221,   308,   307,     0,   146,   185,     0,   189,
     191,   128,   130,   141,     0,   118,   119,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   470,   472,
     474,   476,   478,   480,   482,   468,   214,   371,   370,   499,
     492,   493,   494,   495,   496,   497,   466,   490,   491,     0,
       0,   373,   372,   377,   376,     0,   201,     0,   489,   486,
     487,   488,   379,   218,     0,   484,   383,   382,     0,     0,
     504,   505,   507,   506,   499,   386,   387,   391,   392,   395,
     393,     0,   396,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   510,   511,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   435,   434,   503,   500,   501,
     502,   436,   437,     0,     0,   440,   450,   442,   444,   446,
     448,   452,   451,     0,   459,   455,   456,   457,   461,   460,
     462,   464,   463,     0,     0,     0,    95,    96,    97,    98,
      99,   101,   100,   102,   103,   104,   148,   148,   148,     0,
       0,     0,     0,     0,     0,    15,   280,     0,     0,     0,
     312,   311,   309,   314,   313,   310,   315,   316,     0,     0,
       0,     0,   285,   285,   285,   285,   285,   285,     0,     0,
       0,     0,     0,     0,   285,   285,   285,     0,     0,     0,
       0,     0,     0,     0,     0,   291,   298,     0,     0,     0,
       0,   302,     0,     0,     0,     0,     0,   226,   225,   228,
     229,   230,   231,   232,   227,   233,   234,   236,   235,   237,
     238,   239,   240,   241,   242,   243,   244,     0,     0,     0,
      33,     0,    49,   184,    94,   194,   196,    94,   180,    94,
     160,     0,     0,    94,   162,    94,   164,     0,     0,     0,
       0,     0,   122,   121,   123,   124,   126,   178,    93,    35,
     112,   109,   110,     0,     0,     0,     0,     0,     0,     0,
     485,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,   218,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   165,   167,   174,   171,   149,   172,
     173,   169,   166,   168,   175,   170,   323,     0,   251,   252,
     253,   255,   254,   256,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     248,   249,   250,     0,     0,     0,     0,   277,     0,   292,
     293,   294,     0,   301,   296,   278,     0,   279,   297,   145,
     133,     0,   107,    51,   198,   182,   203,     0,    93,   157,
     207,   211,     0,   186,   284,   188,   190,     0,     0,     0,
      94,   177,   111,   498,     0,     0,     0,     0,     0,     0,
       0,   469,   467,   375,   374,   378,   380,   394,   397,     0,
     508,   509,     0,     0,   514,   512,   465,     0,   406,   431,
     432,   418,   419,     0,   417,   422,   420,   421,   424,   425,
     426,   428,   430,   427,   429,   423,     0,     0,     0,     0,
     405,   403,   404,   402,   438,   454,   258,   286,   267,   268,
     269,   270,   271,   272,   285,   285,   285,   285,   285,   285,
     259,   260,   261,     0,     0,     0,     0,   299,     0,     0,
       0,     0,     0,   138,    93,    94,   154,   317,     0,   317,
     125,   127,   179,   471,   473,   477,   481,   475,   479,   483,
     513,   515,     0,     0,     0,   416,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   274,   275,   276,     0,
     289,     0,   303,   221,   290,   144,   137,     0,    94,   155,
     158,     0,   319,   187,   319,   407,   216,   217,   398,   399,
     400,   401,   273,   262,   263,   264,   265,   266,     0,     0,
       0,   139,    93,     0,   305,   131,   288,   295,   136,     0,
     156,   318,     0,   140,     0,     0,     0,     0,     0,     0,
     321,   320
  };

  const short
  parser::yypgoto_[] =
  {
    -605,  -605,  -605,   -25,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,   -77,  -264,
    -605,  -605,  -605,  -605,  -605,  -314,   412,  -605,  -604,  -605,
    -605,  -605,    52,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
     569,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -189,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -368,  -605,
    -605,  -605,  -605,  -188,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
      90,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
     597,    95,  -119,  -605,    91,  -605,    88,  -605,    87,  -156,
    -605,  -605,   -75,  -605,     6,   -92,  -605,   -15,   -10,  -125,
    -605,  -461,    94,  -605,  -605,  -605,  -605,   596,  -280,  -134,
    -176,   188,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -138,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,
    -605,  -605,  -605,  -605,  -605,  -605,  -605,  -605,  -137,   -19,
      33,  -132,  -605,  -429,  -605,  -605,  -605,  -605,    46,  -605,
    -110,   570,   582,   567,  -147,    53,  -605,  -605
  };

  const short
  parser::yydefgoto_[] =
  {
       0,     1,     2,    96,     6,    17,    29,   285,    18,    19,
      20,    21,    22,   183,   184,   352,   185,   403,    82,    83,
      23,   186,   355,   612,   187,   615,   529,   188,   353,   189,
     404,   641,   642,   190,   191,   192,   193,   194,   195,   196,
     400,   197,   198,   391,   199,    24,    25,    93,   609,   608,
     842,    26,    27,    90,   607,   841,   200,   201,   687,   202,
     203,   204,   205,   748,   206,   207,   208,   209,   210,   211,
     212,   213,   214,   215,   216,   217,   402,   638,   218,   365,
     366,    28,   219,   220,   221,   222,   223,   224,   225,   226,
     357,   358,   435,   367,   368,   374,   375,   379,   380,   415,
     369,   865,   662,   376,    84,    85,   291,   354,    92,   361,
     755,   705,    87,   577,   582,   733,   227,   385,   549,   892,
     914,   228,   229,   230,   231,   232,   233,   234,   235,   236,
     237,   238,   239,   240,   241,   242,   243,   244,   245,   246,
     247,   248,   469,   249,   250,   251,   252,   253,   254,   255,
     256,   257,   258,   259,   260,   261,   262,   263,   264,   265,
     266,   267,   268,   269,   270,   271,   272,   273,   274,   275,
     276,   277,   278,   279,   280,   281,   282,   283,   470,   417,
     418,   454,   420,   421,   422,   423,   424,   425,   650,   445,
     426,   427,   428,   491,   456,   473,   474,   475
  };

  const short
  parser::yytable_[] =
  {
      91,    86,    88,   458,   284,   419,   419,   419,   496,   360,
     360,   419,    94,   370,   551,   377,   381,   747,   482,   483,
     504,   383,   478,   480,   484,  -283,   419,   364,   538,   384,
    -200,  -283,  -283,   477,     3,   664,   455,  -200,   405,   477,
     457,   419,   493,   286,   287,   288,   289,   290,   419,   616,
     419,   618,   664,   620,   494,   497,   503,   498,   476,   499,
     624,   500,   586,  -215,    36,   626,   442,   505,   506,   507,
    -215,   510,  -205,  -219,  -209,  -212,   460,   462,  -213,  -205,
    -219,  -209,  -212,    30,   479,  -213,   448,   449,   481,    31,
      32,   373,   706,   707,   708,   709,   710,   386,   393,    77,
      78,    79,   763,   717,   718,   719,   477,   359,   359,    89,
      -2,   359,   392,   359,   359,   782,   783,   292,   464,   431,
     433,    33,    34,    67,   446,    69,    70,    71,    72,    73,
     436,    75,   477,     4,     5,   728,   477,   924,   925,   485,
     879,   729,  -304,   416,   416,   416,   880,   443,  -304,   416,
     293,   436,   768,   769,   501,   770,   294,   443,   443,   689,
     690,   508,    81,   511,   416,   436,   448,   449,   295,   408,
     416,   432,   434,   863,   864,   408,   447,   296,   297,   416,
     298,   299,   416,   300,   301,   796,   416,   444,   416,   405,
     302,   486,   303,   304,   356,   362,   362,   461,   463,   371,
     305,   378,   382,   487,   488,   489,   502,   490,   495,    95,
     387,   388,   306,   509,   307,   512,    77,    78,    79,    80,
     395,   396,   408,   409,   410,   411,   412,   413,   414,  -283,
     450,   451,   452,   453,  -200,   438,   347,   448,   449,   343,
     344,   345,   408,   308,   346,   309,   310,   439,   440,   441,
     337,   338,   339,   340,   341,   342,   343,   344,   345,   311,
     535,   346,   764,   765,   766,   312,   767,  -215,   408,   397,
     398,   399,   408,   697,   313,   314,  -205,  -219,  -209,  -212,
     315,   316,  -213,   317,   318,   552,   553,   554,   555,   556,
     557,   558,   559,   560,   561,   562,   563,   564,   565,   566,
     319,   320,   408,   409,   410,   411,   412,   413,   414,   321,
     465,   466,   467,   468,   322,   323,   567,   568,   569,   450,
     451,   452,   453,   929,   761,   637,   324,   325,   326,   349,
     351,   613,   348,   587,   588,   589,   590,   591,   592,   593,
     594,   595,   596,   597,   598,   599,   600,   601,   602,   603,
     604,   605,   606,   350,   389,   390,   534,   346,   862,   610,
     537,   611,   614,   870,   871,   872,   873,   874,   875,   617,
     619,   621,   622,   408,   409,   410,   411,   412,   413,   414,
     623,   625,   628,   629,   630,   632,   539,   548,   550,   633,
     634,   636,   639,   640,    36,   340,   341,   342,   343,   344,
     345,   643,   575,   346,   644,   645,   635,   570,   571,   572,
     573,   574,   576,   578,   579,   580,   581,   583,   584,   585,
     513,   514,   515,   516,   517,   518,   519,   520,   521,   522,
     523,   524,   525,   646,   846,   540,   541,   542,   543,   544,
     545,   546,   547,   287,   647,   648,   649,   651,   526,   527,
     654,   655,   665,    67,   652,    69,    70,    71,    72,    73,
     653,    75,   666,   667,   668,   669,   749,   670,   671,   672,
     663,   673,   847,   754,   676,   674,   678,   849,   675,   627,
     677,   679,   680,   681,   682,   683,   698,   663,   631,   756,
     771,   773,    81,   699,   700,   701,   702,   703,   704,   711,
     712,   774,   713,   684,   685,   714,   715,   716,   723,   724,
     725,   688,   688,   688,   726,   798,   800,   727,   528,   691,
     692,   693,   360,   695,   730,   731,   732,   370,   734,   735,
     889,   377,   736,   381,   801,   802,   772,   795,   797,   775,
     364,   805,   737,   812,   813,   738,   741,   810,   811,   739,
     335,   336,   337,   338,   339,   340,   341,   342,   343,   344,
     345,   799,   803,   346,   743,   752,   763,   757,   759,   760,
     764,   886,   814,   815,   807,   779,   776,    36,   780,   777,
     778,   781,   784,   740,   785,   786,   788,   806,   808,   809,
     789,   790,   792,   794,   481,   816,   818,   533,   920,   819,
     820,   742,   821,   822,   823,   830,   831,   832,   839,   686,
     844,   845,   848,   853,   866,   854,   855,   867,   753,   856,
     359,   857,   858,    36,   859,   359,   694,   860,   416,   359,
     861,   359,   868,   869,   876,   416,    67,   877,    69,    70,
      71,    72,    73,   878,    75,   881,    77,    78,    79,    80,
     567,   568,   416,   416,   884,   887,   888,   443,   895,   902,
     443,   443,   891,   903,   899,   901,   919,   922,   904,   905,
     906,   443,   443,   443,   443,    81,   907,   416,   416,   916,
     917,   921,    67,   852,    69,    70,    71,    72,    73,   890,
      75,   926,   927,   762,   817,   930,   931,   910,   401,   363,
     912,   824,   825,   826,   827,   828,   829,   745,   362,   744,
     746,   750,   751,   371,   882,   894,   836,   378,   915,   382,
     394,    81,   696,   471,     0,   804,   840,   492,     0,     0,
       0,   843,   749,     0,   898,   472,   900,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   360,     0,     0,
       0,     0,     0,     0,     0,     0,     0,   758,   540,   541,
     542,   543,   544,   545,   546,   547,   327,   328,   329,   330,
     331,   332,   333,   334,   335,   336,   337,   338,   339,   340,
     341,   342,   343,   344,   345,   568,     0,   346,     0,     0,
       0,   536,     0,   595,   327,   328,   329,   330,   331,   332,
     333,   334,   335,   336,   337,   338,   339,   340,   341,   342,
     343,   344,   345,     0,     0,   346,   885,   833,   834,   835,
       0,     0,   837,     0,     0,     0,   838,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,   893,     0,
       0,     0,     0,     0,     0,   359,     0,     0,     0,     0,
       0,   416,   850,   851,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   908,     0,     0,     0,     7,   896,
     897,     0,     0,     0,     8,     0,   913,   911,     0,     0,
     -41,     0,     0,     0,     0,   918,     0,     0,     0,     0,
       0,     0,     0,     0,   416,     0,     0,     0,     0,     0,
       0,     0,     0,     0,   -41,     0,   -41,     0,     0,   923,
     720,   928,     0,   327,   328,   329,   330,   331,   332,   333,
     334,   335,   336,   337,   338,   339,   340,   341,   342,   343,
     344,   345,     0,   883,   346,     0,     0,     0,     0,     0,
       9,   -41,    10,   530,   531,   532,   516,   517,   518,   519,
     520,   521,   522,   523,   524,   525,     0,   -41,   -41,   -41,
      11,    12,    13,   -41,   -41,   -41,   -41,   -41,   -41,   -41,
       0,   -41,     0,     0,    14,   909,   -41,   -41,   -41,   -41,
     -41,   -41,    15,    16,   -41,   -41,   -41,   -41,   -41,   -41,
       0,   -41,   -41,   -41,   -41,   -41,     0,   -41,   -41,   -41,
     -41,   -41,   -41,   -41,   -41,   -41,     0,     0,     0,     0,
       0,     0,     0,     0,   -41,   -41,   -41,   -41,   -41,   -41,
     -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,
     -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,
     -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,
     -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,   -41,
      35,    36,     0,     0,     0,     0,     0,     0,    37,     0,
       0,   437,     0,     0,     0,     0,     0,   -41,     0,     0,
      39,    40,     0,     0,     0,     0,     0,     0,     0,     0,
       0,    41,     0,     0,    42,    43,     0,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
      57,    58,    59,    60,    61,    62,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
      77,    78,    79,    80,    35,    36,     0,     0,     0,     0,
       0,     0,    37,     0,     0,   437,     0,     0,     0,     0,
       0,     0,     0,     0,    39,    40,     0,     0,     0,    81,
       0,     0,     0,     0,     0,    41,     0,     0,    42,    43,
       0,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,    76,    77,    78,    79,    80,   721,     0,
       0,   327,   328,   329,   330,   331,   332,   333,   334,   335,
     336,   337,   338,   339,   340,   341,   342,   343,   344,   345,
       0,     0,   346,    81,     0,     0,    35,    36,     0,     0,
       0,     0,     0,     0,    37,     0,     0,    38,     0,     0,
       0,     0,   438,     0,     0,     0,    39,    40,     0,     0,
       0,   459,     0,     0,   439,   440,   441,    41,     0,     0,
      42,    43,     0,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,    54,    55,    56,    57,    58,    59,    60,
      61,   656,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,    76,    77,    78,    79,    80,
      35,    36,     0,     0,     0,     0,   405,     0,    37,     0,
       0,    38,     0,     0,     0,     0,   438,     0,     0,     0,
      39,    40,     0,     0,     0,    81,     0,     0,   439,   440,
     441,    41,     0,     0,    42,    43,     0,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
      57,    58,    59,    60,   406,   407,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
      77,    78,    79,    80,   722,     0,     0,   327,   328,   329,
     330,   331,   332,   333,   334,   335,   336,   337,   338,   339,
     340,   341,   342,   343,   344,   345,     0,     0,   346,    81,
      35,    36,     0,     0,     0,     0,     0,     0,    37,     0,
       0,    38,     0,     0,     0,     0,     0,     0,   410,     0,
      39,    40,     0,     0,   657,   658,     0,   659,   660,   661,
       0,    41,     0,     0,    42,    43,     0,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
      57,    58,    59,    60,    61,    62,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
      77,    78,    79,    80,     0,     0,     0,    35,    36,     0,
       0,     0,     0,     0,     0,    37,     0,     0,    38,     0,
     408,   409,   410,   411,   412,   413,   414,    39,    40,    81,
     429,   430,     0,     0,     0,     0,     0,     0,    41,     0,
       0,    42,    43,     0,    44,    45,    46,    47,    48,    49,
      50,    51,    52,    53,    54,    55,    56,    57,    58,    59,
      60,    61,    62,    63,    64,    65,    66,    67,    68,    69,
      70,    71,    72,    73,    74,    75,    76,    77,    78,    79,
      80,     0,   329,   330,   331,   332,   333,   334,   335,   336,
     337,   338,   339,   340,   341,   342,   343,   344,   345,    35,
      36,   346,     0,     0,     0,     0,    81,    37,     0,     0,
      38,     0,     0,     0,     0,     0,     0,     0,     0,    39,
      40,     0,     0,     0,     0,     0,     0,   764,   765,   766,
      41,   767,     0,    42,    43,     0,    44,    45,    46,    47,
      48,    49,    50,    51,    52,    53,    54,    55,    56,    57,
      58,    59,    60,    61,    62,    63,    64,    65,    66,    67,
      68,    69,    70,    71,    72,    73,    74,    75,    76,    77,
      78,    79,    80,     0,     0,     0,    35,    36,     0,     0,
       0,     0,     0,     0,    37,     0,     0,    38,     0,     0,
       0,     0,     0,     0,     0,     0,    39,    40,    81,     0,
       0,     0,     0,     0,     0,   768,   769,    41,   770,     0,
      42,    43,     0,    44,    45,    46,    47,    48,    49,    50,
      51,    52,    53,    54,    55,    56,    57,    58,    59,    60,
      61,    62,    63,    64,    65,    66,    67,    68,    69,    70,
      71,    72,    73,    74,    75,    76,    77,    78,    79,    80,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    35,    36,
       0,     0,     0,     0,     0,    81,    37,     0,     0,    38,
       0,     0,     0,     0,     0,     0,     0,     0,    39,    40,
       0,     0,     0,     0,     0,     0,     0,     0,     0,    41,
     791,     0,    42,    43,     0,    44,    45,    46,    47,    48,
      49,    50,    51,    52,    53,    54,    55,    56,    57,    58,
      59,    60,    61,    62,    63,    64,    65,    66,    67,    68,
      69,    70,    71,    72,    73,    74,    75,    76,    77,    78,
      79,    80,     0,    35,    36,     0,     0,     0,     0,   405,
       0,    37,     0,     0,    38,     0,     0,     0,     0,     0,
       0,     0,     0,    39,    40,     0,     0,    81,     0,     0,
       0,     0,     0,     0,    41,     0,   793,    42,    43,     0,
      44,    45,    46,    47,    48,    49,    50,    51,    52,    53,
      54,    55,    56,    57,    58,    59,    60,   406,   407,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    76,    77,    78,    79,    80,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      35,    36,     0,     0,     0,     0,   464,     0,    37,     0,
       0,    38,    81,     0,     0,     0,     0,     0,     0,     0,
      39,    40,     0,     0,     0,     0,     0,     0,     0,     0,
       0,    41,     0,     0,    42,    43,   768,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
      57,    58,    59,    60,   406,   407,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
      77,    78,    79,    80,     0,     0,     0,     0,     0,     0,
      35,    36,     0,     0,     0,     0,     0,     0,    37,     0,
       0,    38,     0,     0,     0,     0,     0,     0,     0,    81,
      39,    40,     0,   408,   409,   410,   411,   412,   413,   414,
       0,    41,     0,     0,    42,    43,     0,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
      57,    58,    59,    60,    61,   656,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
      77,    78,    79,    80,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    97,     0,    98,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,    81,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     408,   409,   410,   411,   412,   413,   414,     0,     0,     0,
      99,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,   100,   101,   102,     0,
       0,     0,   103,   104,   105,   106,   107,   108,   109,     0,
     110,     0,     0,     0,     0,   111,   112,   113,   114,   115,
     116,     0,     0,   117,   118,   119,   120,   121,   122,     0,
     123,   124,   125,   126,   127,     0,   128,   129,   130,   131,
     132,   133,   134,   135,   136,     0,     0,     0,     0,     0,
       0,     0,   410,   137,   138,   139,   140,   141,   142,   143,
     144,   145,   146,   147,   148,   149,   150,   151,   152,   153,
     154,   155,   156,   157,   158,   159,   160,   161,   162,   163,
     164,   165,   166,   167,   168,   169,   170,   171,   172,   173,
     174,   175,   176,   177,   178,   179,   180,   181,   182,    35,
      36,     0,     0,     0,     0,     0,     0,    37,     0,     0,
      38,     0,     0,     0,     0,     0,     0,     0,     0,    39,
      40,     0,     0,     0,     0,     0,     0,     0,     0,     0,
      41,     0,     0,    42,    43,   372,    44,    45,    46,    47,
      48,    49,    50,    51,    52,    53,    54,    55,    56,    57,
      58,    59,    60,    61,    62,    63,    64,    65,    66,    67,
      68,    69,    70,    71,    72,    73,    74,    75,    76,    77,
      78,    79,    80,    35,    36,     0,     0,     0,     0,     0,
       0,    37,     0,     0,    38,     0,     0,     0,     0,     0,
       0,     0,     0,    39,    40,     0,     0,     0,    81,     0,
       0,     0,     0,     0,    41,     0,     0,    42,    43,     0,
      44,    45,    46,    47,    48,    49,    50,    51,    52,    53,
      54,    55,    56,    57,    58,    59,    60,    61,    62,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    76,    77,    78,    79,    80,   327,   328,   329,
     330,   331,   332,   333,   334,   335,   336,   337,   338,   339,
     340,   341,   342,   343,   344,   345,     0,     0,   346,     0,
       0,     0,    81,   137,   138,   139,   140,   141,   142,   143,
     144,   145,   146,   147,   148,   149,   150,   151,   152,   153,
     154,   155,   156,   157,   158,   159,   160,   161,   162,   163,
     164,   165,   166,   167,   168,   169,   170,   171,   172,   173,
     174,   175,   176,   177,   178,   179,   180,   181,   182,   327,
     328,   329,   330,   331,   332,   333,   334,   787,   336,   337,
     338,   339,   340,   341,   342,   343,   344,   345,     0,   327,
     346,   329,   330,   331,   332,   333,   334,   335,   336,   337,
     338,   339,   340,   341,   342,   343,   344,   345,     0,     0,
     346
  };

  const short
  parser::yycheck_[] =
  {
      15,    11,    12,   150,    29,   137,   138,   139,   164,   101,
     102,   143,    13,   105,   294,   107,   108,   621,   156,   156,
     176,   103,   154,   155,   156,     6,   158,   104,   292,   111,
       6,    12,    13,    57,     0,   464,   146,    13,     9,    57,
     150,   173,   161,    37,    38,    39,    40,    41,   180,   363,
     182,   365,   481,   367,    14,   165,   175,   167,     9,   169,
     374,   171,   326,     6,     4,   379,   141,   177,   178,   179,
      13,   181,     6,     6,     6,     6,   151,   152,     6,    13,
      13,    13,    13,     1,     9,    13,    57,    58,     9,     7,
       8,   106,   553,   554,   555,   556,   557,   112,   123,    73,
      74,    75,    10,   564,   565,   566,    57,   101,   102,    13,
       0,   105,   122,   107,   108,    23,    24,    11,     9,   138,
     139,     7,     8,    63,   143,    65,    66,    67,    68,    69,
     140,    71,    57,    23,    24,     6,    57,    38,    39,   158,
       6,    12,     6,   137,   138,   139,    12,   141,    12,   143,
      11,   161,   201,   202,   173,   204,    11,   151,   152,   527,
     528,   180,   102,   182,   158,   175,    57,    58,    11,   193,
     164,   138,   139,    23,    24,   193,   143,    11,    11,   173,
      11,    11,   176,    11,    11,   203,   180,   141,   182,     9,
      11,   158,    11,    11,   100,   101,   102,   151,   152,   105,
      11,   107,   108,   200,   201,   202,   173,   204,   162,   210,
     116,   117,    11,   180,    11,   182,    73,    74,    75,    76,
     126,   127,   193,   194,   195,   196,   197,   198,   199,   210,
     201,   202,   203,   204,   210,   195,    13,    57,    58,    31,
      32,    33,   193,    11,    36,    11,    11,   207,   208,   209,
      25,    26,    27,    28,    29,    30,    31,    32,    33,    11,
     285,    36,   200,   201,   202,    11,   204,   210,   193,   127,
     128,   129,   193,   537,    11,    11,   210,   210,   210,   210,
      11,    11,   210,    11,    11,   295,   296,   297,   298,   299,
     300,   301,   302,   303,   304,   305,   306,   307,   308,   309,
      11,    11,   193,   194,   195,   196,   197,   198,   199,    11,
     201,   202,   203,   204,    11,    11,   310,   311,   312,   201,
     202,   203,   204,   927,   638,   402,    11,    11,    11,    74,
      74,   356,    13,   327,   328,   329,   330,   331,   332,   333,
     334,   335,   336,   337,   338,   339,   340,   341,   342,   343,
     344,   345,   346,    13,    74,    74,     8,    36,   787,    74,
      11,     6,     6,   824,   825,   826,   827,   828,   829,     6,
       6,     9,     6,   193,   194,   195,   196,   197,   198,   199,
       6,     6,     6,     6,     6,     6,   292,   293,   294,     6,
       6,   401,    74,     4,     4,    28,    29,    30,    31,    32,
      33,   204,    12,    36,    11,    11,   400,   313,   314,   315,
     316,   317,   318,   319,   320,   321,   322,   323,   324,   325,
      77,    78,    79,    80,    81,    82,    83,    84,    85,    86,
      87,    88,    89,     6,   748,   139,   140,   141,   142,   143,
     144,   145,   146,   437,     6,     6,     6,     6,   105,   106,
       6,     6,     6,    63,    11,    65,    66,    67,    68,    69,
      11,    71,     6,     6,     6,     6,   622,     6,     6,     6,
     464,     6,   752,   629,     6,   204,     6,   757,    11,   385,
     204,     6,     6,     6,     6,     6,    12,   481,   394,    74,
     646,   647,   102,    12,    12,    12,    12,    12,     6,     6,
       6,   648,     6,   513,   514,     6,     6,     6,     6,     6,
       6,   526,   527,   528,     6,   671,   672,    12,   175,   529,
     530,   531,   614,   533,    12,    12,     6,   619,    12,    12,
     844,   623,     6,   625,   672,   672,   646,   669,   670,   649,
     617,   673,    12,   681,   681,    12,     6,   679,   680,    13,
      23,    24,    25,    26,    27,    28,    29,    30,    31,    32,
      33,   671,   672,    36,     4,     6,    10,     6,     6,     6,
     200,    13,   682,   683,     9,    11,   651,     4,    10,   654,
     655,    10,    10,   608,    10,    10,    10,    10,    10,     9,
     665,   666,   667,   668,     9,    12,    12,   185,   912,    12,
      12,   611,    12,    12,    12,    12,    12,    12,     6,   515,
      10,     6,     6,    12,     6,    12,    12,   204,   628,    12,
     614,    12,    12,     4,    12,   619,   532,    10,   622,   623,
      10,   625,     6,   204,    12,   629,    63,    12,    65,    66,
      67,    68,    69,    12,    71,     6,    73,    74,    75,    76,
     644,   645,   646,   647,    12,     6,     6,   651,    10,    12,
     654,   655,     9,    12,    10,    10,     6,     6,    12,    12,
      12,   665,   666,   667,   668,   102,    12,   671,   672,    12,
      12,    10,    63,   760,    65,    66,    67,    68,    69,   845,
      71,     9,     9,   641,   704,    10,    10,   886,   129,   102,
     888,   711,   712,   713,   714,   715,   716,   617,   614,   614,
     619,   623,   625,   619,   839,   849,   726,   623,   894,   625,
     124,   102,   534,   153,    -1,   672,   736,   160,    -1,    -1,
      -1,   741,   888,    -1,   866,   153,   868,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,   839,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,     6,   139,   140,
     141,   142,   143,   144,   145,   146,    15,    16,    17,    18,
      19,    20,    21,    22,    23,    24,    25,    26,    27,    28,
      29,    30,    31,    32,    33,   779,    -1,    36,    -1,    -1,
      -1,    12,    -1,   787,    15,    16,    17,    18,    19,    20,
      21,    22,    23,    24,    25,    26,    27,    28,    29,    30,
      31,    32,    33,    -1,    -1,    36,   841,   723,   724,   725,
      -1,    -1,   728,    -1,    -1,    -1,   732,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   848,    -1,
      -1,    -1,    -1,    -1,    -1,   839,    -1,    -1,    -1,    -1,
      -1,   845,   758,   759,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,   879,    -1,    -1,    -1,     1,   863,
     864,    -1,    -1,    -1,     7,    -1,   891,   887,    -1,    -1,
      13,    -1,    -1,    -1,    -1,   910,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,   888,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    37,    -1,    39,    -1,    -1,   919,
      12,   926,    -1,    15,    16,    17,    18,    19,    20,    21,
      22,    23,    24,    25,    26,    27,    28,    29,    30,    31,
      32,    33,    -1,   839,    36,    -1,    -1,    -1,    -1,    -1,
      73,    74,    75,    77,    78,    79,    80,    81,    82,    83,
      84,    85,    86,    87,    88,    89,    -1,    90,    91,    92,
      93,    94,    95,    96,    97,    98,    99,   100,   101,   102,
      -1,   104,    -1,    -1,   107,   881,   109,   110,   111,   112,
     113,   114,   115,   116,   117,   118,   119,   120,   121,   122,
      -1,   124,   125,   126,   127,   128,    -1,   130,   131,   132,
     133,   134,   135,   136,   137,   138,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,   147,   148,   149,   150,   151,   152,
     153,   154,   155,   156,   157,   158,   159,   160,   161,   162,
     163,   164,   165,   166,   167,   168,   169,   170,   171,   172,
     173,   174,   175,   176,   177,   178,   179,   180,   181,   182,
     183,   184,   185,   186,   187,   188,   189,   190,   191,   192,
       3,     4,    -1,    -1,    -1,    -1,    -1,    -1,    11,    -1,
      -1,    14,    -1,    -1,    -1,    -1,    -1,   210,    -1,    -1,
      23,    24,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    34,    -1,    -1,    37,    38,    -1,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,    76,     3,     4,    -1,    -1,    -1,    -1,
      -1,    -1,    11,    -1,    -1,    14,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    23,    24,    -1,    -1,    -1,   102,
      -1,    -1,    -1,    -1,    -1,    34,    -1,    -1,    37,    38,
      -1,    40,    41,    42,    43,    44,    45,    46,    47,    48,
      49,    50,    51,    52,    53,    54,    55,    56,    57,    58,
      59,    60,    61,    62,    63,    64,    65,    66,    67,    68,
      69,    70,    71,    72,    73,    74,    75,    76,    12,    -1,
      -1,    15,    16,    17,    18,    19,    20,    21,    22,    23,
      24,    25,    26,    27,    28,    29,    30,    31,    32,    33,
      -1,    -1,    36,   102,    -1,    -1,     3,     4,    -1,    -1,
      -1,    -1,    -1,    -1,    11,    -1,    -1,    14,    -1,    -1,
      -1,    -1,   195,    -1,    -1,    -1,    23,    24,    -1,    -1,
      -1,   204,    -1,    -1,   207,   208,   209,    34,    -1,    -1,
      37,    38,    -1,    40,    41,    42,    43,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
      57,    58,    59,    60,    61,    62,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
       3,     4,    -1,    -1,    -1,    -1,     9,    -1,    11,    -1,
      -1,    14,    -1,    -1,    -1,    -1,   195,    -1,    -1,    -1,
      23,    24,    -1,    -1,    -1,   102,    -1,    -1,   207,   208,
     209,    34,    -1,    -1,    37,    38,    -1,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,    76,    12,    -1,    -1,    15,    16,    17,
      18,    19,    20,    21,    22,    23,    24,    25,    26,    27,
      28,    29,    30,    31,    32,    33,    -1,    -1,    36,   102,
       3,     4,    -1,    -1,    -1,    -1,    -1,    -1,    11,    -1,
      -1,    14,    -1,    -1,    -1,    -1,    -1,    -1,   195,    -1,
      23,    24,    -1,    -1,   201,   202,    -1,   204,   205,   206,
      -1,    34,    -1,    -1,    37,    38,    -1,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,    76,    -1,    -1,    -1,     3,     4,    -1,
      -1,    -1,    -1,    -1,    -1,    11,    -1,    -1,    14,    -1,
     193,   194,   195,   196,   197,   198,   199,    23,    24,   102,
     203,   204,    -1,    -1,    -1,    -1,    -1,    -1,    34,    -1,
      -1,    37,    38,    -1,    40,    41,    42,    43,    44,    45,
      46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
      56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
      76,    -1,    17,    18,    19,    20,    21,    22,    23,    24,
      25,    26,    27,    28,    29,    30,    31,    32,    33,     3,
       4,    36,    -1,    -1,    -1,    -1,   102,    11,    -1,    -1,
      14,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    23,
      24,    -1,    -1,    -1,    -1,    -1,    -1,   200,   201,   202,
      34,   204,    -1,    37,    38,    -1,    40,    41,    42,    43,
      44,    45,    46,    47,    48,    49,    50,    51,    52,    53,
      54,    55,    56,    57,    58,    59,    60,    61,    62,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    76,    -1,    -1,    -1,     3,     4,    -1,    -1,
      -1,    -1,    -1,    -1,    11,    -1,    -1,    14,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    23,    24,   102,    -1,
      -1,    -1,    -1,    -1,    -1,   201,   202,    34,   204,    -1,
      37,    38,    -1,    40,    41,    42,    43,    44,    45,    46,
      47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
      57,    58,    59,    60,    61,    62,    63,    64,    65,    66,
      67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,     3,     4,
      -1,    -1,    -1,    -1,    -1,   102,    11,    -1,    -1,    14,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    23,    24,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    34,
     204,    -1,    37,    38,    -1,    40,    41,    42,    43,    44,
      45,    46,    47,    48,    49,    50,    51,    52,    53,    54,
      55,    56,    57,    58,    59,    60,    61,    62,    63,    64,
      65,    66,    67,    68,    69,    70,    71,    72,    73,    74,
      75,    76,    -1,     3,     4,    -1,    -1,    -1,    -1,     9,
      -1,    11,    -1,    -1,    14,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    23,    24,    -1,    -1,   102,    -1,    -1,
      -1,    -1,    -1,    -1,    34,    -1,   203,    37,    38,    -1,
      40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
      50,    51,    52,    53,    54,    55,    56,    57,    58,    59,
      60,    61,    62,    63,    64,    65,    66,    67,    68,    69,
      70,    71,    72,    73,    74,    75,    76,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
       3,     4,    -1,    -1,    -1,    -1,     9,    -1,    11,    -1,
      -1,    14,   102,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      23,    24,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    34,    -1,    -1,    37,    38,   201,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,    76,    -1,    -1,    -1,    -1,    -1,    -1,
       3,     4,    -1,    -1,    -1,    -1,    -1,    -1,    11,    -1,
      -1,    14,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   102,
      23,    24,    -1,   193,   194,   195,   196,   197,   198,   199,
      -1,    34,    -1,    -1,    37,    38,    -1,    40,    41,    42,
      43,    44,    45,    46,    47,    48,    49,    50,    51,    52,
      53,    54,    55,    56,    57,    58,    59,    60,    61,    62,
      63,    64,    65,    66,    67,    68,    69,    70,    71,    72,
      73,    74,    75,    76,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    37,    -1,    39,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   102,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
     193,   194,   195,   196,   197,   198,   199,    -1,    -1,    -1,
      74,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    90,    91,    92,    -1,
      -1,    -1,    96,    97,    98,    99,   100,   101,   102,    -1,
     104,    -1,    -1,    -1,    -1,   109,   110,   111,   112,   113,
     114,    -1,    -1,   117,   118,   119,   120,   121,   122,    -1,
     124,   125,   126,   127,   128,    -1,   130,   131,   132,   133,
     134,   135,   136,   137,   138,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,   195,   147,   148,   149,   150,   151,   152,   153,
     154,   155,   156,   157,   158,   159,   160,   161,   162,   163,
     164,   165,   166,   167,   168,   169,   170,   171,   172,   173,
     174,   175,   176,   177,   178,   179,   180,   181,   182,   183,
     184,   185,   186,   187,   188,   189,   190,   191,   192,     3,
       4,    -1,    -1,    -1,    -1,    -1,    -1,    11,    -1,    -1,
      14,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    23,
      24,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      34,    -1,    -1,    37,    38,    39,    40,    41,    42,    43,
      44,    45,    46,    47,    48,    49,    50,    51,    52,    53,
      54,    55,    56,    57,    58,    59,    60,    61,    62,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      74,    75,    76,     3,     4,    -1,    -1,    -1,    -1,    -1,
      -1,    11,    -1,    -1,    14,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    23,    24,    -1,    -1,    -1,   102,    -1,
      -1,    -1,    -1,    -1,    34,    -1,    -1,    37,    38,    -1,
      40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
      50,    51,    52,    53,    54,    55,    56,    57,    58,    59,
      60,    61,    62,    63,    64,    65,    66,    67,    68,    69,
      70,    71,    72,    73,    74,    75,    76,    15,    16,    17,
      18,    19,    20,    21,    22,    23,    24,    25,    26,    27,
      28,    29,    30,    31,    32,    33,    -1,    -1,    36,    -1,
      -1,    -1,   102,   147,   148,   149,   150,   151,   152,   153,
     154,   155,   156,   157,   158,   159,   160,   161,   162,   163,
     164,   165,   166,   167,   168,   169,   170,   171,   172,   173,
     174,   175,   176,   177,   178,   179,   180,   181,   182,   183,
     184,   185,   186,   187,   188,   189,   190,   191,   192,    15,
      16,    17,    18,    19,    20,    21,    22,    23,    24,    25,
      26,    27,    28,    29,    30,    31,    32,    33,    -1,    15,
      36,    17,    18,    19,    20,    21,    22,    23,    24,    25,
      26,    27,    28,    29,    30,    31,    32,    33,    -1,    -1,
      36
  };

  const short
  parser::yystos_[] =
  {
       0,   212,   213,     0,    23,    24,   215,     1,     7,    73,
      75,    93,    94,    95,   107,   115,   116,   216,   219,   220,
     221,   222,   223,   231,   256,   257,   262,   263,   292,   217,
       1,     7,     8,     7,     8,     3,     4,    11,    14,    23,
      24,    34,    37,    38,    40,    41,    42,    43,    44,    45,
      46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
      56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
      66,    67,    68,    69,    70,    71,    72,    73,    74,    75,
      76,   102,   229,   230,   315,   316,   319,   323,   319,    13,
     264,   318,   319,   258,    13,   210,   214,    37,    39,    74,
      90,    91,    92,    96,    97,    98,    99,   100,   101,   102,
     104,   109,   110,   111,   112,   113,   114,   117,   118,   119,
     120,   121,   122,   124,   125,   126,   127,   128,   130,   131,
     132,   133,   134,   135,   136,   137,   138,   147,   148,   149,
     150,   151,   152,   153,   154,   155,   156,   157,   158,   159,
     160,   161,   162,   163,   164,   165,   166,   167,   168,   169,
     170,   171,   172,   173,   174,   175,   176,   177,   178,   179,
     180,   181,   182,   183,   184,   185,   186,   187,   188,   189,
     190,   191,   192,   224,   225,   227,   232,   235,   238,   240,
     244,   245,   246,   247,   248,   249,   250,   252,   253,   255,
     267,   268,   270,   271,   272,   273,   275,   276,   277,   278,
     279,   280,   281,   282,   283,   284,   285,   286,   289,   293,
     294,   295,   296,   297,   298,   299,   300,   327,   332,   333,
     334,   335,   336,   337,   338,   339,   340,   341,   342,   343,
     344,   345,   346,   347,   348,   349,   350,   351,   352,   354,
     355,   356,   357,   358,   359,   360,   361,   362,   363,   364,
     365,   366,   367,   368,   369,   370,   371,   372,   373,   374,
     375,   376,   377,   378,   379,   380,   381,   382,   383,   384,
     385,   386,   387,   388,   214,   218,   315,   315,   315,   315,
     315,   317,    11,    11,    11,    11,    11,    11,    11,    11,
      11,    11,    11,    11,    11,    11,    11,    11,    11,    11,
      11,    11,    11,    11,    11,    11,    11,    11,    11,    11,
      11,    11,    11,    11,    11,    11,    11,    15,    16,    17,
      18,    19,    20,    21,    22,    23,    24,    25,    26,    27,
      28,    29,    30,    31,    32,    33,    36,    13,    13,    74,
      13,    74,   226,   239,   318,   233,   323,   301,   302,   315,
     316,   320,   323,   301,   229,   290,   291,   304,   305,   311,
     316,   323,    39,   318,   306,   307,   314,   316,   323,   308,
     309,   316,   323,   103,   111,   328,   318,   323,   323,    74,
      74,   254,   319,   214,   328,   323,   323,   127,   128,   129,
     251,   251,   287,   228,   241,     9,    57,    58,   193,   194,
     195,   196,   197,   198,   199,   310,   315,   390,   391,   392,
     393,   394,   395,   396,   397,   398,   401,   402,   403,   203,
     204,   390,   391,   390,   391,   303,   319,    14,   195,   207,
     208,   209,   313,   315,   399,   400,   390,   391,    57,    58,
     201,   202,   203,   204,   392,   401,   405,   401,   405,   204,
     313,   399,   313,   399,     9,   201,   202,   203,   204,   353,
     389,   402,   403,   406,   407,   408,     9,    57,   392,     9,
     392,     9,   353,   389,   392,   390,   391,   200,   201,   202,
     204,   404,   404,   303,    14,   399,   310,   401,   401,   401,
     401,   390,   391,   303,   310,   401,   401,   401,   390,   391,
     401,   390,   391,    77,    78,    79,    80,    81,    82,    83,
      84,    85,    86,    87,    88,    89,   105,   106,   175,   237,
      77,    78,    79,   237,     8,   214,    12,    11,   230,   323,
     139,   140,   141,   142,   143,   144,   145,   146,   323,   329,
     323,   329,   319,   319,   319,   319,   319,   319,   319,   319,
     319,   319,   319,   319,   319,   319,   319,   315,   315,   315,
     323,   323,   323,   323,   323,    12,   323,   324,   323,   323,
     323,   323,   325,   323,   323,   323,   230,   315,   315,   315,
     315,   315,   315,   315,   315,   315,   315,   315,   315,   315,
     315,   315,   315,   315,   315,   315,   315,   265,   260,   259,
      74,     6,   234,   214,     6,   236,   236,     6,   236,     6,
     236,     9,     6,     6,   236,     6,   236,   323,     6,     6,
       6,   323,     6,     6,     6,   315,   319,   229,   288,    74,
       4,   242,   243,   204,    11,    11,     6,     6,     6,     6,
     399,     6,    11,    11,     6,     6,    58,   201,   202,   204,
     205,   206,   313,   315,   394,     6,     6,     6,     6,     6,
       6,     6,     6,     6,   204,    11,     6,   204,     6,     6,
       6,     6,     6,     6,   319,   319,   323,   269,   318,   269,
     269,   319,   319,   319,   323,   319,   332,   230,    12,    12,
      12,    12,    12,    12,     6,   322,   322,   322,   322,   322,
     322,     6,     6,     6,     6,     6,     6,   322,   322,   322,
      12,    12,    12,     6,     6,     6,     6,    12,     6,    12,
      12,    12,     6,   326,    12,    12,     6,    12,    12,    13,
     214,     6,   319,     4,   302,   291,   305,   239,   274,   310,
     307,   309,     6,   319,   310,   321,    74,     6,     6,     6,
       6,   236,   243,    10,   200,   201,   202,   204,   201,   202,
     204,   310,   401,   310,   405,   401,   313,   313,   313,    11,
      10,    10,    23,    24,    10,    10,    10,    23,    10,   313,
     313,   204,   313,   203,   313,   392,   203,   392,   310,   401,
     310,   353,   389,   401,   406,   392,    10,     9,    10,     9,
     392,   392,   353,   389,   401,   401,    12,   319,    12,    12,
      12,    12,    12,    12,   319,   319,   319,   319,   319,   319,
      12,    12,    12,   323,   323,   323,   319,   323,   323,     6,
     319,   266,   261,   319,    10,     6,   236,   329,     6,   329,
     323,   323,   229,    12,    12,    12,    12,    12,    12,    12,
      10,    10,   394,    23,    24,   312,     6,   204,     6,   204,
     322,   322,   322,   322,   322,   322,    12,    12,    12,     6,
      12,     6,   320,   323,    12,   214,    13,     6,     6,   236,
     310,     9,   330,   319,   330,    10,   315,   315,   392,    10,
     392,    10,    12,    12,    12,    12,    12,    12,   318,   323,
     260,   319,   274,   318,   331,   331,    12,    12,   214,     6,
     236,    10,     6,   319,    38,    39,     9,     9,   318,   239,
      10,    10
  };

  const short
  parser::yyr1_[] =
  {
       0,   211,   212,   213,   213,   214,   214,   215,   215,   215,
     216,   216,   217,   216,   218,   216,   219,   219,   219,   219,
     219,   219,   219,   219,   220,   221,   222,   223,   223,   223,
     223,   224,   226,   225,   228,   227,   229,   229,   229,   230,
     230,   231,   231,   231,   231,   231,   231,   231,   233,   232,
     234,   234,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   235,   235,   235,   235,   235,   235,   235,
     235,   235,   235,   236,   236,   237,   237,   237,   237,   237,
     237,   237,   237,   237,   237,   238,   239,   239,   241,   240,
     242,   242,   243,   244,   245,   246,   247,   248,   249,   250,
     251,   251,   251,   251,   252,   252,   252,   252,   253,   254,
     254,   255,   255,   256,   258,   259,   257,   260,   261,   261,
     261,   262,   264,   265,   263,   266,   267,   268,   269,   269,
     270,   271,   272,   273,   273,   273,   273,   274,   274,   275,
     275,   276,   276,   277,   277,   278,   279,   280,   280,   280,
     280,   281,   282,   283,   284,   285,   287,   286,   288,   288,
     289,   290,   290,   291,   292,   293,   293,   293,   294,   295,
     295,   296,   297,   298,   299,   300,   300,   301,   301,   302,
     302,   303,   304,   304,   305,   305,   306,   306,   307,   307,
     308,   308,   309,   309,   310,   311,   312,   312,   313,   314,
     315,   315,   316,   316,   316,   316,   316,   316,   316,   316,
     316,   316,   316,   316,   316,   316,   316,   316,   316,   316,
     316,   316,   316,   316,   316,   316,   316,   316,   316,   316,
     316,   316,   316,   316,   316,   316,   316,   317,   316,   316,
     316,   316,   316,   316,   316,   316,   316,   316,   316,   316,
     316,   316,   316,   316,   316,   316,   316,   316,   316,   316,
     316,   318,   319,   320,   321,   322,   322,   323,   323,   323,
     323,   323,   323,   323,   323,   323,   323,   323,   324,   324,
     325,   325,   326,   326,   326,   327,   328,   328,   328,   329,
     329,   329,   329,   329,   329,   329,   329,   330,   330,   331,
     331,   331,   332,   332,   333,   333,   333,   333,   333,   333,
     333,   333,   333,   333,   333,   333,   333,   333,   333,   333,
     333,   333,   333,   333,   333,   333,   333,   333,   333,   333,
     333,   333,   333,   333,   333,   333,   333,   333,   333,   333,
     333,   333,   333,   333,   333,   333,   333,   333,   333,   333,
     334,   334,   335,   335,   335,   335,   336,   336,   337,   338,
     338,   339,   340,   340,   341,   342,   343,   343,   344,   345,
     346,   347,   347,   348,   348,   348,   349,   349,   350,   350,
     351,   351,   352,   352,   352,   352,   353,   353,   354,   354,
     354,   354,   354,   354,   354,   354,   355,   355,   356,   356,
     357,   357,   358,   359,   360,   360,   361,   361,   361,   361,
     361,   362,   362,   363,   364,   364,   365,   366,   367,   368,
     368,   369,   370,   371,   372,   373,   374,   375,   376,   377,
     378,   379,   379,   380,   381,   382,   383,   384,   385,   385,
     386,   386,   387,   388,   388,   389,   390,   390,   391,   391,
     392,   392,   393,   393,   394,   394,   395,   395,   396,   396,
     397,   397,   398,   398,   399,   399,   400,   400,   400,   400,
     401,   401,   402,   402,   402,   402,   402,   402,   402,   403,
     404,   404,   404,   404,   405,   405,   405,   405,   406,   406,
     406,   406,   407,   407,   408,   408
  };

  const signed char
  parser::yyr2_[] =
  {
       0,     2,     1,     0,     3,     1,     1,     0,     1,     1,
       2,     1,     0,     3,     0,     4,     1,     1,     1,     1,
       1,     1,     1,     1,     3,     3,     2,     1,     2,     2,
       2,     1,     0,     3,     0,     3,     1,     1,     1,     1,
       1,     0,     1,     1,     2,     2,     2,     2,     0,     3,
       0,     2,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     0,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     2,     1,     3,     0,     3,
       1,     2,     1,     1,     1,     1,     1,     1,     2,     2,
       0,     2,     2,     2,     3,     5,     3,     5,     2,     0,
       1,     7,     1,     5,     0,     0,     9,     0,     1,     3,
       5,     3,     0,     0,     7,     0,     2,     1,     0,     1,
       1,     1,     1,     2,     5,     6,     8,     1,     3,     1,
       3,     1,     3,     1,     3,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     3,     3,     0,     4,     1,     3,
       3,     1,     3,     1,     4,     2,     4,     6,     4,     2,
       4,     2,     1,     1,     3,     1,     3,     1,     3,     1,
       1,     1,     1,     3,     1,     1,     1,     3,     1,     1,
       1,     3,     1,     1,     1,     1,     2,     2,     1,     1,
       1,     1,     1,     1,     2,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     3,     2,     2,     2,     4,     4,
       4,     4,     4,     4,     4,     4,     4,     0,     5,     5,
       5,     5,     7,     7,     7,     7,     7,     5,     5,     5,
       5,     5,     5,     7,     6,     6,     6,     4,     4,     4,
       3,     1,     1,     1,     1,     0,     2,     1,     8,     6,
       6,     3,     4,     4,     4,     8,     4,     4,     1,     3,
       0,     2,     0,     3,     3,     7,     0,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     0,     3,     0,
       6,     6,     1,     3,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
       2,     2,     2,     2,     4,     4,     2,     2,     4,     2,
       4,     1,     2,     2,     1,     1,     2,     2,     1,     1,
       1,     2,     2,     2,     4,     2,     2,     4,     6,     6,
       6,     6,     4,     4,     4,     4,     3,     5,     1,     1,
       1,     1,     1,     1,     1,     1,     5,     4,     4,     4,
       4,     4,     4,     4,     4,     4,     4,     4,     4,     4,
       4,     4,     4,     1,     2,     2,     2,     2,     4,     1,
       2,     1,     2,     1,     2,     1,     2,     1,     2,     1,
       2,     2,     2,     1,     4,     2,     2,     2,     1,     2,
       2,     2,     2,     2,     2,     3,     1,     3,     1,     3,
       1,     4,     1,     4,     1,     4,     1,     4,     1,     4,
       1,     4,     1,     4,     1,     2,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     3,     1,
       1,     1,     1,     1,     1,     1,     1,     1,     3,     3,
       1,     1,     3,     4,     3,     4
  };




#if YYDEBUG
  const short
  parser::yyrline_[] =
  {
       0,   312,   312,   315,   316,   319,   319,   322,   323,   328,
     338,   339,   341,   341,   349,   349,   371,   372,   373,   374,
     375,   376,   378,   379,   383,   394,   412,   431,   432,   433,
     434,   438,   444,   444,   453,   453,   464,   467,   470,   476,
     479,   485,   486,   489,   492,   495,   498,   502,   509,   509,
     518,   521,   528,   529,   530,   531,   532,   533,   534,   535,
     536,   537,   538,   539,   540,   541,   542,   543,   544,   545,
     546,   547,   548,   549,   550,   551,   552,   553,   554,   555,
     556,   557,   558,   559,   560,   561,   562,   563,   564,   565,
     566,   567,   568,   571,   571,   574,   577,   580,   583,   586,
     589,   592,   595,   598,   601,   607,   613,   622,   641,   641,
     648,   649,   653,   659,   665,   671,   677,   683,   689,   695,
     701,   704,   707,   710,   716,   723,   730,   734,   741,   751,
     754,   758,   761,   767,   774,   776,   774,   785,   791,   796,
     801,   809,   816,   818,   816,   827,   833,   839,   845,   848,
     852,   858,   864,   870,   873,   876,   882,   891,   894,   901,
     904,   908,   911,   915,   918,   922,   928,   934,   937,   940,
     943,   949,   957,   965,   973,   979,   985,   985,   995,   998,
    1004,  1007,  1008,  1012,  1018,  1026,  1031,  1036,  1044,  1050,
    1053,  1059,  1065,  1071,  1076,  1079,  1083,  1090,  1091,  1095,
    1098,  1105,  1115,  1116,  1120,  1123,  1132,  1133,  1137,  1140,
    1149,  1150,  1154,  1157,  1166,  1173,  1180,  1184,  1192,  1199,
    1206,  1209,  1218,  1221,  1224,  1228,  1231,  1234,  1237,  1240,
    1243,  1246,  1249,  1252,  1255,  1258,  1261,  1264,  1267,  1270,
    1273,  1276,  1279,  1282,  1285,  1288,  1291,  1295,  1299,  1303,
    1307,  1310,  1314,  1317,  1320,  1323,  1326,  1329,  1329,  1335,
    1338,  1341,  1344,  1347,  1350,  1353,  1356,  1359,  1362,  1365,
    1368,  1371,  1374,  1377,  1380,  1383,  1388,  1393,  1396,  1399,
    1402,  1408,  1416,  1422,  1428,  1434,  1437,  1447,  1450,  1456,
    1462,  1468,  1471,  1474,  1478,  1482,  1485,  1488,  1504,  1507,
    1514,  1515,  1522,  1523,  1527,  1534,  1540,  1543,  1546,  1552,
    1555,  1558,  1561,  1564,  1567,  1570,  1573,  1579,  1582,  1592,
    1597,  1602,  1611,  1612,  1616,  1617,  1618,  1619,  1620,  1621,
    1622,  1623,  1624,  1625,  1626,  1627,  1628,  1629,  1630,  1631,
    1632,  1633,  1634,  1635,  1636,  1637,  1638,  1639,  1640,  1641,
    1642,  1643,  1644,  1645,  1646,  1647,  1648,  1649,  1650,  1651,
    1652,  1653,  1654,  1655,  1656,  1657,  1658,  1659,  1660,  1661,
    1665,  1669,  1675,  1679,  1682,  1685,  1692,  1696,  1702,  1709,
    1713,  1720,  1726,  1730,  1736,  1742,  1748,  1751,  1757,  1763,
    1769,  1775,  1778,  1784,  1788,  1792,  1798,  1802,  1809,  1812,
    1818,  1821,  1827,  1833,  1839,  1842,  1848,  1849,  1857,  1858,
    1859,  1860,  1861,  1862,  1863,  1864,  1868,  1872,  1879,  1882,
    1889,  1893,  1900,  1906,  1912,  1916,  1925,  1929,  1932,  1935,
    1938,  1945,  1949,  1958,  1964,  1968,  1974,  1980,  1986,  1993,
    1996,  2002,  2008,  2015,  2021,  2028,  2034,  2041,  2047,  2054,
    2060,  2070,  2074,  2080,  2086,  2093,  2100,  2107,  2114,  2118,
    2125,  2129,  2135,  2142,  2146,  2154,  2160,  2161,  2167,  2170,
    2178,  2179,  2183,  2184,  2188,  2189,  2193,  2194,  2198,  2199,
    2203,  2204,  2208,  2209,  2213,  2214,  2220,  2223,  2226,  2229,
    2234,  2234,  2237,  2240,  2243,  2246,  2249,  2252,  2255,  2261,
    2267,  2270,  2273,  2276,  2282,  2285,  2288,  2291,  2297,  2300,
    2303,  2306,  2312,  2313,  2317,  2318
  };

  void
  parser::yy_stack_print_ () const
  {
    *yycdebug_ << "Stack now";
    for (stack_type::const_iterator
           i = yystack_.begin (),
           i_end = yystack_.end ();
         i != i_end; ++i)
      *yycdebug_ << ' ' << int (i->state);
    *yycdebug_ << '\n';
  }

  void
  parser::yy_reduce_print_ (int yyrule) const
  {
    int yylno = yyrline_[yyrule];
    int yynrhs = yyr2_[yyrule];
    // Print the symbols being reduced, and their result.
    *yycdebug_ << "Reducing stack by rule " << yyrule - 1
               << " (line " << yylno << "):\n";
    // The symbols being reduced.
    for (int yyi = 0; yyi < yynrhs; yyi++)
      YY_SYMBOL_PRINT ("   $" << yyi + 1 << " =",
                       yystack_[(yynrhs) - (yyi + 1)]);
  }
#endif // YYDEBUG


} // yy
#line 5702 "src/asm/parser.cpp"

#line 2321 "src/asm/parser.y"


// Semantic actions.

void yy::parser::error(std::string const &str) {
	::error("%s\n", str.c_str());
}

static uint32_t str2int2(std::vector<uint8_t> const &s) {
	uint32_t length = s.size();

	if (length > 4)
		warning(
		    WARNING_NUMERIC_STRING_1,
		    "Treating string as a number ignores first %" PRIu32 " character%s\n",
		    length - 4,
		    length == 5 ? "" : "s"
		);
	else if (length > 1)
		warning(
		    WARNING_NUMERIC_STRING_2, "Treating %" PRIu32 "-character string as a number\n", length
		);

	uint32_t r = 0;

	for (uint32_t i = length < 4 ? 0 : length - 4; i < length; i++) {
		r <<= 8;
		r |= s[i];
	}

	return r;
}

static void errorInvalidUTF8Byte(uint8_t byte, char const *functionName) {
	error("%s: Invalid UTF-8 byte 0x%02hhX\n", functionName, byte);
}

static size_t strlenUTF8(std::string const &str) {
	char const *ptr = str.c_str();
	size_t len = 0;
	uint32_t state = 0;

	for (uint32_t codep = 0; *ptr; ptr++) {
		uint8_t byte = *ptr;

		switch (decode(&state, &codep, byte)) {
		case 1:
			errorInvalidUTF8Byte(byte, "STRLEN");
			state = 0;
			// fallthrough
		case 0:
			len++;
			break;
		}
	}

	// Check for partial code point.
	if (state != 0)
		error("STRLEN: Incomplete UTF-8 character\n");

	return len;
}

static std::string strsubUTF8(std::string const &str, uint32_t pos, uint32_t len) {
	char const *ptr = str.c_str();
	size_t index = 0;
	uint32_t state = 0;
	uint32_t codep = 0;
	uint32_t curPos = 1; // RGBASM strings are 1-indexed!

	// Advance to starting position in source string.
	while (ptr[index] && curPos < pos) {
		switch (decode(&state, &codep, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSUB");
			state = 0;
			// fallthrough
		case 0:
			curPos++;
			break;
		}
		index++;
	}

	// A position 1 past the end of the string is allowed, but will trigger the
	// "Length too big" warning below if the length is nonzero.
	if (!ptr[index] && pos > curPos)
		warning(
		    WARNING_BUILTIN_ARG, "STRSUB: Position %" PRIu32 " is past the end of the string\n", pos
		);

	size_t startIndex = index;
	uint32_t curLen = 0;

	// Compute the result length in bytes.
	while (ptr[index] && curLen < len) {
		switch (decode(&state, &codep, ptr[index])) {
		case 1:
			errorInvalidUTF8Byte(ptr[index], "STRSUB");
			state = 0;
			// fallthrough
		case 0:
			curLen++;
			break;
		}
		index++;
	}

	if (curLen < len)
		warning(WARNING_BUILTIN_ARG, "STRSUB: Length too big: %" PRIu32 "\n", len);

	// Check for partial code point.
	if (state != 0)
		error("STRSUB: Incomplete UTF-8 character\n");

	return std::string(ptr + startIndex, ptr + index);
}

static size_t charlenUTF8(std::string const &str) {
	std::string_view view = str;
	size_t len;

	for (len = 0; charmap_ConvertNext(view, nullptr); len++)
		;

	return len;
}

static std::string charsubUTF8(std::string const &str, uint32_t pos) {
	std::string_view view = str;
	size_t charLen = 1;

	// Advance to starting position in source string.
	for (uint32_t curPos = 1; charLen && curPos < pos; curPos++)
		charLen = charmap_ConvertNext(view, nullptr);

	std::string_view start = view;

	if (!charmap_ConvertNext(view, nullptr))
		warning(
		    WARNING_BUILTIN_ARG,
		    "CHARSUB: Position %" PRIu32 " is past the end of the string\n",
		    pos
		);

	start = start.substr(0, start.length() - view.length());
	return std::string(start);
}

static uint32_t adjustNegativePos(int32_t pos, size_t len, char const *functionName) {
	// STRSUB and CHARSUB adjust negative `pos` arguments the same way,
	// such that position -1 is the last character of a string.
	if (pos < 0)
		pos += len + 1;
	if (pos < 1) {
		warning(WARNING_BUILTIN_ARG, "%s: Position starts at 1\n", functionName);
		pos = 1;
	}
	return (uint32_t)pos;
}

static std::string strrpl(std::string_view str, std::string const &old, std::string const &rep) {
	if (old.empty()) {
		warning(WARNING_EMPTY_STRRPL, "STRRPL: Cannot replace an empty string\n");
		return std::string(str);
	}

	std::string rpl;

	while (!str.empty()) {
		auto pos = str.find(old);
		if (pos == str.npos) {
			rpl.append(str);
			break;
		}
		rpl.append(str, 0, pos);
		rpl.append(rep);
		str.remove_prefix(pos + old.size());
	}

	return rpl;
}

static std::string strfmt(
    std::string const &spec,
    std::vector<std::variant<uint32_t, std::string>> const &args
) {
	std::string str;
	size_t argIndex = 0;

	for (size_t i = 0; spec[i] != '\0'; ++i) {
		int c = spec[i];

		if (c != '%') {
			str += c;
			continue;
		}

		c = spec[++i];

		if (c == '%') {
			str += c;
			continue;
		}

		FormatSpec fmt{};

		while (c != '\0') {
			fmt.useCharacter(c);
			if (fmt.isFinished())
				break;
			c = spec[++i];
		}

		if (fmt.isEmpty()) {
			error("STRFMT: Illegal '%%' at end of format string\n");
			str += '%';
			break;
		}

		if (!fmt.isValid()) {
			error("STRFMT: Invalid format spec for argument %zu\n", argIndex + 1);
			str += '%';
		} else if (argIndex >= args.size()) {
			// Will warn after formatting is done.
			str += '%';
		} else if (auto *n = std::get_if<uint32_t>(&args[argIndex]); n) {
			fmt.appendNumber(str, *n);
		} else {
			assume(std::holds_alternative<std::string>(args[argIndex]));
			auto &s = std::get<std::string>(args[argIndex]);
			fmt.appendString(str, s);
		}

		argIndex++;
	}

	if (argIndex < args.size())
		error("STRFMT: %zu unformatted argument(s)\n", args.size() - argIndex);
	else if (argIndex > args.size())
		error(
		    "STRFMT: Not enough arguments for format spec, got: %zu, need: %zu\n",
		    args.size(),
		    argIndex
		);

	return str;
}

static void compoundAssignment(std::string const &symName, RPNCommand op, int32_t constValue) {
	Expression oldExpr, constExpr, newExpr;
	int32_t newValue;

	oldExpr.makeSymbol(symName);
	constExpr.makeNumber(constValue);
	newExpr.makeBinaryOp(op, std::move(oldExpr), constExpr);
	newValue = newExpr.getConstVal();
	sym_AddVar(symName, newValue);
}

static void failAssert(AssertionType type) {
	switch (type) {
	case ASSERT_FATAL:
		fatalerror("Assertion failed\n");
	case ASSERT_ERROR:
		error("Assertion failed\n");
		break;
	case ASSERT_WARN:
		warning(WARNING_ASSERT, "Assertion failed\n");
		break;
	}
}

static void failAssertMsg(AssertionType type, std::string const &message) {
	switch (type) {
	case ASSERT_FATAL:
		fatalerror("Assertion failed: %s\n", message.c_str());
	case ASSERT_ERROR:
		error("Assertion failed: %s\n", message.c_str());
		break;
	case ASSERT_WARN:
		warning(WARNING_ASSERT, "Assertion failed: %s\n", message.c_str());
		break;
	}
}
